@@ -33,8 +33,7 @@ from pyodide_mkdocs_theme.pyodide_macros.html_dependencies.deps_class import Dep
 from .. import html_builder as Html
 from ..tools_and_constants import HtmlClass, IdeConstants, IdeMode, PmtPyMacrosName, Prefix
 from ..messages import Tip
-from ..parsing import admonition_safe_html, build_code_fence, items_comma_joiner
-from ..paths_utils import convert_url_to_utf8, to_uri
+from ..parsing import add_indent, admonition_safe_html, build_code_fence, items_comma_joiner
 from ..plugin.tools.test_cases import Case
 from ..plugin.tools.macros_data import IdeToTest
 from ..plugin.config import PLUGIN_CONFIG_SRC
@@ -71,7 +70,7 @@ class Ide(CommonTermIde):
 
     KEEP_CORR_ON_EXPORT_TO_JS: ClassVar[bool] = True
 
-    MODE: ClassVar[ Literal["","_v"] ] = ""
+    IDE_VERT: ClassVar[ Literal["","_v"] ] = ""
     """ The terminal will be below (mode="") or on the right (mode="_v") of the editor.
         (what an awful interface, yeah... x) )
     """
@@ -247,7 +246,7 @@ class Ide(CommonTermIde):
             and self.env.deactivate_stdout_for_secrets
             and not self.std_key
             and any( 'terminal_message' in section for section in (
-                self.files_data.public_tests,  self.files_data.secret_tests,
+                self.files_data.tests_content,  self.files_data.secrets_content,
             ))
         ):
             msg += self._build_error_msg_with_option(
@@ -286,10 +285,6 @@ class Ide(CommonTermIde):
                     )
                 )
 
-        elif self.profile not in IdeMode.VALUES:
-            msg += f'Invalid MODE argument: { self.profile !r} (choices: { ", ".join(IdeMode.VALUES) })'
-
-
         elif self.profile == IdeMode.delayed_reveal:
             bad = [*filter(bool,[
                 "\n    No `tests` section should be present."   * self.has_tests,
@@ -305,6 +300,21 @@ class Ide(CommonTermIde):
 
 
 
+    def _get_corr_rems_msg(self, present:bool=True):
+        """ Build error messages for files validations during the mkdocs build.  """
+        elements = [*filter(bool,(
+            "a correction"      * (present == self.has_corr),
+            "a rem content"     * (present == self.has_rem),
+            "a vis_rem content" * (present == self.has_vis_rem),
+        ))]
+        elt_msg = items_comma_joiner(elements, 'and')
+        single  = len(elements)==1
+        verb    = f"exist{ 's' * (single)}" if present else f"{ 'is' if single else 'are' } missing"
+        elt_msg = f"{ elt_msg } { verb }".capitalize()
+        return elt_msg
+
+
+
     def exported_items(self):
         yield from super().exported_items()
         yield from [
@@ -316,15 +326,14 @@ class Ide(CommonTermIde):
             ("has_corr_btn",    self.has_corr_btn),
             ("has_reveal_btn",  self.has_reveal_btn),
             ("has_counter",     self.has_counter),
-            ("is_vert",         self.MODE == '_v'),
+            ("is_vert",         self.IDE_VERT == '_v'),
             ("max_ide_lines",   self.max_size),
             ("min_ide_lines",   self.min_size),
             ('profile',         self.profile or ""),    # HAS to be exported => ensure is not None
-            ("src_hash",        self._get_src_hash()),
+            ("src_hash",        self._content_changes_tracker_src_hash()),
             ("two_cols",        self.two_cols),
             ("std_key",         self.std_key),
 
-            ("is_encrypted",                           self.env.encrypt_corrections_and_rems),
             ("deactivate_stdout_for_secrets",          self.env.deactivate_stdout_for_secrets),
             ("decrease_attempts_on_user_code_failure", self.env.decrease_attempts_on_user_code_failure),    # pylint: disable=line-too-long
             ("show_only_assertion_errors_for_secrets", self.env.show_only_assertion_errors_for_secrets),    # pylint: disable=line-too-long
@@ -332,71 +341,23 @@ class Ide(CommonTermIde):
 
 
 
-    def _get_corr_rems_msg(self, present:bool=True):
-        elements = [*filter(bool,(
-            "a correction"   * (present == self.has_corr),
-            "a REM file"     * (present == self.has_rem),
-            "a VIS_REM file" * (present == self.has_vis_rem),
-        ))]
-        elt_msg = items_comma_joiner(elements, 'and')
-        single  = len(elements)==1
-        verb    = f"exist{ 's' * (single)}" if present else f"{ 'is' if single else 'are' } missing"
-        elt_msg = f"{ elt_msg } { verb }".capitalize()
-        return elt_msg
-
-
-
-    def _get_src_hash(self):
-        """ Generate the sha256 hash of `code + TestToken + tests` """
-        content = (
+    def _content_changes_tracker_src_hash(self):
+        """
+        Generate the sha256 hash of `code + TestToken + tests` (helper to know when to warn the
+        user about a content update since the last time they used that IDE on the website).
+        """
+        to_hash = '\n'.join((
             self.files_data.user_content,
             self.env.lang.tests.msg,
             self.files_data.public_tests
-        )
+        ))
         h = sha256()
-        h.update( bytes('\n'.join(content), encoding='utf-8') )
+        h.update( bytes(to_hash, encoding='utf-8') )
         return h.hexdigest()
 
 
 
 
-    def generate_id(self):
-        """
-        Generate an id number for the current IDE (editor+terminal), as a "prefix_hash(32bits)".
-
-        This id must be:
-            - Unique to every IDE used throughout the whole website.
-            - Stable, so that it can be used to identify what IDE goes with what file or what
-              localStorage data.
-
-        Current strategy:
-            - If the file exists, hash its path.
-            - If there is no file, use the current global IDE_counter and hash its value as string.
-            - The "mode" of the IDE is appended to the string before hashing.
-            - Any ID value (macro argument) is also appended to the string before hashing.
-
-        Uniqueness of the resulting hash is verified and a BuildError is raised if two identical
-        hashes are encountered.
-        """
-        py_path = self.files_data.exo_py
-        if py_path:
-            path = str(py_path)
-        else:
-            path = str(self.env.ide_count)
-
-        if self.MODE:
-            path += self.MODE       # legacy...
-
-        path_without_id = path
-
-        if self.id is not None:
-            path += str(self.id)
-
-        return self.id_to_hash(path, path_without_id)
-
-
-
-
     def make_element(self) -> str:
         """
         Create an IDE (Editor+Terminal+buttons) within an Mkdocs document. {py_name}.py
@@ -410,17 +371,17 @@ class Ide(CommonTermIde):
         solution_div = self.build_corr_and_rems()
 
         return f"{ global_layout }{ solution_div }\n\n"
-            # `solution_div` is not inside the other because it causes markdown rendering troubles
-            # otherwise (because of "md_in_html").
-            # Also it will become useful as "anchor point" for IDE extractions and insertions for
-            # full screen and split screen modes.
+            # `solution_div` is not inside the other because it would cause markdown rendering
+            # troubles with "md_in_html".
+            # Also it became useful as "anchor point" for IDE extractions and insertions in the
+            # DOM, for full screen and split screen modes.
             #
             # NOTE: about indentations: global_layout + the beginning of solution_div is a unique,
             #       long-ass string of html only, so everything is still properly indented when it
             #       comes to markdown rendering.
             #
             # NOTE: DON'T EVER PUT NEW LINES AT THE BEGINNING!!! (breaks indentation contract: the
-            #       macro call itself is not indented properly)
+            #       macro call itself would not indented properly anymore)
 
 
 
@@ -428,7 +389,7 @@ class Ide(CommonTermIde):
         """
         Generate the global layout that will receive later the ace elements.
         """
-        is_v = self.MODE == '_v'
+        is_v = self.IDE_VERT == '_v'
         tip: Tip = self.env.lang.comments
         msg = str(tip)
 
@@ -436,7 +397,7 @@ class Ide(CommonTermIde):
         editor_div = Html.div(
             id = self.editor_name,
             is_v = str(is_v).lower(),
-            mode = self.MODE,
+            mode = self.IDE_VERT,
         )
         shortcut_comment_asserts = Html.span(
             '###' + Html.tooltip(msg, tip.em, shift=95),
@@ -445,6 +406,7 @@ class Ide(CommonTermIde):
         )
         display_modes_btns_div = ''.join(
             f'<div class="{ kls } { HtmlClass.tooltip } { HtmlClass.svg_switch_btn } twemoji">{ svg + Html.tooltip(str(tip), tip.em, shift=95) }</div>'
+                # The twemoji class allows to use :material-icons: in the UI
             for kls,svg,tip in [
                 ('ide-split-screen', SVG_FULL_SCREEN, self.env.lang.split_screen),
                 ('ide-full-screen', SVG_SPLIT_SCREEN, self.env.lang.full_screen),
@@ -458,14 +420,14 @@ class Ide(CommonTermIde):
 
         terminal_div = self.make_terminal(
             Prefix.term_ + self.editor_name ,
-            kls = f"{ HtmlClass.term_editor }{ self.MODE }",
+            kls = f"{ HtmlClass.term_editor }{ self.IDE_VERT }",
             n_lines_h = self.term_height * (not is_v),
             is_v = is_v,
         )
 
         ide_and_term = Html.div(
             f"{ editor_wrapper }{ terminal_div }",
-            kls = f"{ HtmlClass.py_mk_wrapper }{ self.MODE } md-typeset",
+            kls = f"{ HtmlClass.py_mk_wrapper }{ self.IDE_VERT } md-typeset",
                 # Adding md-typeset class for split-screen mode display consistency
         )
 
@@ -528,6 +490,7 @@ class Ide(CommonTermIde):
             (str(self.env.lang.title_corr), self.has_corr),
             (str(self.env.lang.title_rem),  self.has_rem)
         )))
+        one_level = '    '
         corr_content = self.files_data.corr_content
         at_least_one = self.has_corr or self.has_rem
         anything     = at_least_one or self.has_vis_rem
@@ -537,8 +500,8 @@ class Ide(CommonTermIde):
 
         # Build the whole div content:
         md_div = [         '',   # Extra empty line to enforce proper rendering of the md around
-                           f'<div markdown="1" id="{ Prefix.solution_ }{ self.editor_name }" '
-                           f'     class="{ HtmlClass.py_mk_hidden }" data-search-exclude >',
+                           f'<div markdown="1" id="{ Prefix.solution_ }{ self.editor_name }"'
+                           f' class="{ HtmlClass.py_mk_hidden }" data-search-exclude >',        # CONTINUATION!
                             *extra_tokens ]
         if at_least_one:
             md_div.append( f'??? tip "{ sol_title }"' )
@@ -549,7 +512,6 @@ class Ide(CommonTermIde):
         if self.has_corr:
             # Inner indented content must be handled now when building the block. The indentation
             # for the current line
-            one_level = '    '
             fence = build_code_fence(
                 corr_content,
                 one_level + self.indentation,
@@ -559,19 +521,24 @@ class Ide(CommonTermIde):
 
         if self.has_corr and self.has_rem:
             rem = self.env.lang.rem
-            md_div.append( f'    <span class="{ HtmlClass.rem_fake_h3 }">{ rem } :</span>')
+            md_div.append( f'{ one_level }<span class="{ HtmlClass.rem_fake_h3 }">{ rem } :</span>')
 
         if self.has_rem:
-            rem = self._rem_inclusion('rem_rel_path')
-            md_div.append( f'    { rem }' )
+            rem = self._rem_inclusion('rem_content')
+            # REM md must be "pre-indented" with the final indentation level, except for its first
+            # line which only gets the relative indent for the admonition of corr/REM (see lower:
+            # `md_div` and `out`).
+            indented_rem = add_indent(rem, one_level+self.indentation)
+            md_div.append(one_level+indented_rem)
 
         if self.has_vis_rem:
-            vis_rem = self._rem_inclusion('vis_rem_rel_path')
-            md_div.append(  vis_rem )
+            vis_rem = self._rem_inclusion('vis_rem_content')
+            indented_vis_rem = add_indent(vis_rem, self.indentation)    # Must also indent the content itself
+            md_div.append(indented_vis_rem)
 
         md_div.extend((     *extra_tokens,
-                            '</div>\n\n',
-                      ))    # The extra linefeed is there to enforce rendering of next md sections
+                            '</div>\n\n',   # Extra linefeed to enforce rendering of next md sections
+                      ))
 
         # Add extra indentation according to IDE's macro call position:
         if self.indentation:
@@ -583,10 +550,9 @@ class Ide(CommonTermIde):
 
 
 
-    def _rem_inclusion(self, rem_path_kind:str):
-        path_str = str(getattr(self.files_data, rem_path_kind))
-        rem_uri  = to_uri( convert_url_to_utf8(path_str) )
-        return f'--8<-- "{ rem_uri }"'
+    def _rem_inclusion(self, rem_kind:str):
+        # Dedicated method to allow some dirty hacks in some places... :p
+        return getattr(self.files_data, rem_kind)
 
 
 
@@ -656,4 +622,4 @@ class Ide(CommonTermIde):
 class IdeV(Ide):
 
     MACRO_NAME: ClassVar[PmtPyMacrosName] = PmtPyMacrosName.IDEv
-    MODE: ClassVar[ Literal["","_v"] ] = "_v"
+    IDE_VERT: ClassVar[ Literal["","_v"] ] = "_v"
