@@ -23,23 +23,38 @@ If not, see <https://www.gnu.org/licenses/>.
 from abc import ABCMeta
 import re
 import hashlib
-from typing import Any, ClassVar, Dict, List, Optional, Tuple, Union, TYPE_CHECKING
+from typing import Any, ClassVar, Dict, List, Literal, Optional, Tuple, Union, TYPE_CHECKING
 from dataclasses import dataclass
 from pathlib import Path
 
-from mkdocs.exceptions import BuildError
+from pyodide_mkdocs_theme.pyodide_macros.parsing import items_comma_joiner
 
 
 from .. import html_builder as Html
-from ..pyodide_logger import logger
-from ..exceptions import PyodideMacrosNonUniqueIdError
-from ..tools_and_constants import KEYWORDS_SEPARATOR, PYTHON_KEYWORDS, HtmlClass, IdeConstants, PmtPyMacrosName, SequentialFilter
+from ..exceptions import (
+    PmtInternalError,
+    PmtMacrosError,
+    PmtMacrosInvalidArgumentError,
+    PmtMacrosNonUniqueIdError,
+    PmtMermaidConfigError
+)
+from ..tools_and_constants import (
+    KEYWORDS_SEPARATOR,
+    PYTHON_KEYWORDS,
+    RUN_GROUP_SKIP,
+    HashPathMode,
+    HtmlClass,
+    IdeConstants,
+    P5BtnLocation,
+    PmtPyMacrosName,
+    SequentialFilter,
+)
 from ..messages import Tip
 from ..paths_utils import get_ide_button_png_path
 from ..html_dependencies.deps_class import DepKind
-from ..plugin.tools.maestro_tools import macro_name_to_src_and_is_tester
 from ..plugin.tools.pages_and_macros_py_configs import MacroPyConfig
-from .ide_files_data import IdeFilesExtractor
+from ..plugin.config.definitions.plugin_config import PLUGIN_CONFIG_SRC
+from .ide_files_data import CompositeFilesDataExtractor
 
 if TYPE_CHECKING:
     from ..plugin import PyodideMacrosPlugin
@@ -97,6 +112,9 @@ class IdeManagerMacroArguments:
     Register the kind of js scripts that must be added to the page, for the current object.
     """
 
+    IDE_VERT: ClassVar[ Literal["","_v"] ] = ""
+    """ For IDEs only, but defined here to allow better html id generation """
+
 
     # Defined on instantiation:
     #--------------------------
@@ -105,12 +123,14 @@ class IdeManagerMacroArguments:
     env: 'PyodideMacrosPlugin'
     """ The MaestroEnv singleton """
 
-    py_name: str
-    """ Base name for the files to use (first argument passed to the macros)
-        Partial path from the directory holding the sujet.md file, to the one holding all the
-        other required files, ending with the common prefix for the exercice.
-        Ex:   "exo" to extract:   "exo.py", "exo_corr.py", "exo_test.py", ...
-                "sub_exA/exo" for:  "sub_exA/exo.py", "sub_exA/exo_corr.py", ...
+    py_names: Tuple[str]
+    """
+    Base names for the files to use (first argument passed to the macros), possibly with
+    combinations instructions.
+    Partial path from the directory holding the sujet.md file, to the one holding all the
+    other required files, ending with the common prefix for the exercice.
+    Ex:     "exo" to extract:   "exo.py", "exo_corr.py", "exo_test.py", ...
+            "sub_exA/exo" for:  "sub_exA/exo.py", "sub_exA/exo_corr.py", ...
     """
 
     id: Optional[int]
@@ -175,6 +195,19 @@ class IdeManagerMacroArguments:
     # defined during post_init or in child class
     #-------------------------------------------
 
+    exo_py: Optional[Path] = None
+    """
+    Unresolved absolute path to the target python file.
+    As for PMT 5+, this has to be stored in the IdeManager to be allowed to build the editor html id
+    while the CompositeFilesDataExtractor instances are now cached and unique considering the resolved path.
+    """
+
+    py_name: str = ""
+    """
+    If some python files are given through py_names, this is the stem of the file.
+    Warning: not usable in __post_init__ yet (especially in handle_extra_args).
+    """
+
     built_py_name: str = ""
     """
     Extended python file name, prepending with page url data and stuff, to make the name
@@ -206,7 +239,7 @@ class IdeManagerMacroArguments:
             self.env.set_current_page_insertion_needs(DepKind.mermaid)
 
             if not self.env.is_mermaid_available:
-                raise BuildError(
+                raise PmtMermaidConfigError(
                     "\nCannot use MERMAID=True because the superfences markdown extension is not "
                     "configured to accept mermaid code blocks.\n"
                     "Please add the following in your mkdocs.yml file, in the markdown_extension "
@@ -243,10 +276,10 @@ class IdeManagerMacroArguments:
                 setattr(self, prop, value)
 
         if self.extra_kw:
-            raise BuildError(
-                f"Found forbidden arguments:\n" + "".join(
+            raise PmtMacrosInvalidArgumentError(
+                f"Invalid macro argument:\n" + "".join(
                     f"    {k} = {v!r}\n" for k,v in self.extra_kw.items()
-                ) + f"\n{self}"
+                ) + f"\n{ self.env.log() }"
             )
 
 
@@ -273,30 +306,47 @@ class IdeSectionsManager(IdeManagerMacroArguments):
     relayed to the files_data object.
     """
 
-    files_data: IdeFilesExtractor = None
+    files_data: CompositeFilesDataExtractor = None
 
-    @property               # pylint: disable-next=all
+    # vvvvvvvvv
+    # GENERATED
+    @property
+    def has_env(self): return "env" in self.files_data.contents
+    @property
+    def has_env_term(self): return "env_term" in self.files_data.contents
+    @property
+    def has_code(self): return "code" in self.files_data.contents
+    @property
+    def has_corr(self): return "corr" in self.files_data.contents
+    @property
+    def has_tests(self): return "tests" in self.files_data.contents
+    @property
+    def has_secrets(self): return "secrets" in self.files_data.contents
+    @property
+    def has_post_term(self): return "post_term" in self.files_data.contents
+    @property
+    def has_post(self): return "post" in self.files_data.contents
+    @property
+    def has_rem(self): return "REM" in self.files_data.contents
+    @property
+    def has_vis_rem(self): return "VIS_REM" in self.files_data.contents
+    # GENERATED
+    # ^^^^^^^^^
+
+    @property                                   # pylint: disable-next=all
     def has_any_corr_rems(self):
         return self.has_corr or self.has_rem or self.has_vis_rem
 
-    @property               # pylint: disable-next=all
-    def has_check_btn(self): return False
-
-    def __getattr__(self, prop:str):
-        """
-        Implement all `has_xxx` undefined properties, relaying to the inner IdeFilesExtractor
-        object.
-        """
-        if not prop.startswith('has_'):
-            raise AttributeError(prop)
-        return getattr(self.files_data, prop)
-
+    @property                                   # pylint: disable-next=all
+    def has_check_btn(self):
+        return False
 
 
     def __post_init__(self):
         super().__post_init__()
 
-        self.files_data = IdeFilesExtractor(self.env, self.py_name)
+        self.exo_py, self.files_data = CompositeFilesDataExtractor.get_file_extractor_and_exo_py_for(self.env, self.py_names)
+        self.py_name = self.exo_py.stem if self.exo_py else ""
 
         self._define_max_attempts_symbols_and_value()       # To do before files validation: MAX
 
@@ -306,11 +356,16 @@ class IdeSectionsManager(IdeManagerMacroArguments):
             self.rec_limit = -1
 
         if -1 < self.rec_limit < IdeConstants.min_recursion_limit:
-            raise BuildError(
-                f"The recursion limit for {self} is set too low and may causes runtime troubles. "
-                f"Please set it to at least { IdeConstants.min_recursion_limit }."
+            raise PmtMacrosInvalidArgumentError(
+                f"The recursion limit is set too low and may causes runtime troubles. "
+                f"Please set it to at least { IdeConstants.min_recursion_limit }.{ self.env.log() }"
             )
 
+        # REM contents won't be visible at page load time, hence mkdocs material might miss the requirement.
+        # So, make sure the script will actually be loaded:
+        if "```mermaid" in self.files_data.rem_content + self.files_data.vis_rem_content:
+            self.env.set_current_page_insertion_needs(DepKind.mermaid)
+
 
     def _define_max_attempts_symbols_and_value(self):
         """ Placeholder, to insert (very...) specific logic for IDEs... """
@@ -319,9 +374,19 @@ class IdeSectionsManager(IdeManagerMacroArguments):
     def _validate_files_config(self):
         raise NotImplementedError()
 
+    def _check_forbidden_sections(self, sections:str, err_header:str):
+        forbidden = [
+            section for section in sections.split() if getattr(self, f"has_{ section.lower() }")
+        ]
+        if forbidden:
+            self._validation_outcome(
+                f"{ err_header }, but found:\n"
+                f"    { items_comma_joiner(forbidden, 'and') }"
+            )
+
 
     def _build_error_msg_with_option(self, msg:Optional[str], config_opt:Optional[str]=None):
-        msg = f"\nInvalid configuration with: {self}\n    {msg}"
+        msg = f"\nInvalid configuration with:\n{ msg }\n{ self.env.log() }"
         if config_opt:
             msg += (
                 f"\n    You can deactivate this check by setting `mkdocs.yml:plugins.{config_opt}:"
@@ -333,16 +398,11 @@ class IdeSectionsManager(IdeManagerMacroArguments):
 
     def _validation_outcome(self, msg:Optional[str]):
         """
-        Routine that can be called from the _validate_files_config implementation, handling how
-        the messages must be used (raising/logging).
+        Routine that can be called from the _validate_files_config implementation, raising if given
+        an error message as argument.
         """
-        if not msg:
-            return
-
-        if self.env._dev_mode and 'STD_KEY' not in msg:     # pylint: disable=protected-access
-            logger.error("DEV_MODE (expected x3) - " + msg)
-        else:
-            raise BuildError(msg)
+        if msg:
+            raise PmtMacrosError(f'{ msg }\n{self.env.log()}')
 
 
 
@@ -367,7 +427,7 @@ class IdeManagerMdHtmlGenerator(IdeSectionsManager):
     """
 
     editor_name: str = ''
-    """ tail part of most ids, in the shape of 'editor_{32 bits hexadecimal}' """
+    """ tail part of most html elements' ids, in the shape of 'editor_{32 bits hexadecimal}' """
 
 
     def __post_init__(self):
@@ -388,61 +448,117 @@ class IdeManagerMdHtmlGenerator(IdeSectionsManager):
         """
         Generate an id number for the current element, in the form:
 
-            PREFIX_{32 bits hash value}
+            "{ self.PREFIX_ }{ 32 chars hex value }"
 
-        This id must be:
-            - Unique to every IDE used throughout the whole website.
-            - Stable, so that it can be used to identify what IDE goes with what file or what
-              localStorage data.
+        <br><br>
 
-        Current strategy:
-            - If the file exists, hash its path.
-            - If there is no file, use the current global IDE_counter and hash its value as string.
-            - The "mode" of the IDE is appended to the string before hashing.
-            - Any ID value (macro argument) is also appended to the string before hashing.
+        ## CONSTRAINTS:
 
-        Uniqueness of the resulting hash is verified and a BuildError is raised if two identical
+        - Unique to every runner used throughout the whole website.
+        - Stable, so that it can be used to identify what IDE goes with what file or what
+        localStorage data.
+        - For IDEs:
+            * It must be possible to differentiate IDEs from different pages using the same
+            base python file
+            * It must be possible to identify unambiguously IDEs in different pages that do
+            not use any python file.
+
+        <br><br>
+
+        ## STRATEGIES:
+
+            NOTE: The cache as been deactivated manually and isn't accessible to the user
+                  anymore, but the implementation still makes reference to it, if ever I put
+                  it back in place, one day... (unlikely: removed because mainly useless).
+
+        The string to hash is built in the following ways:
+        - For elements with python file(s):
+            * use the absolute path (unresolved) to the main side fail (first argument if
+              `py_names` is a varargs).
+            * non IDE macros see the macro name added to the string (to differentiate them
+              from IDEs, while keeping backward compatibility for IDEs' ids)
+            * the ID argument _will_ often be needed to differentiate the runners/macros,
+              while allowing cache capabilities without much ambiguity when the user is
+              adding, deleting or moving macro calls around in the md page.
+              This can be turned down with `build.activate_cache: false`: then all non IDE
+              macros fall back to the behavior below.
+            * IDEs specific: the "mode" (/_v) is appended to the string before hashing (legacy).
+
+        - For elements without a python file, or for non IDEs with `build.activate_cache: false`:
+            * use the page url, with the macro name, then a counter of that kind of macro calls.
+
+        Uniqueness of the resulting hash is verified and BuildError is raised if two identical
         hashes are encountered.
-
-        NOTE: uniqueness most be guaranteed for IDEs (LocalStorage). It's less critical for other
-        elements, but they still need to stay unique across a page, at least (especially when
-        feedback is involved, like with terminals. Note: maybe not anymore... :thinking: )
         """
-        path = path_without_id = str(self.env.generic_count)
-        if self.id is not None:
-            path += str(self.id)            # kept in case unlucky collision... (yeah, proba... XD )
-        return self.id_to_hash(path, path_without_id)
+        is_ide = PmtPyMacrosName.is_ide(self.MACRO_NAME)
+
+        if self.exo_py and (is_ide or self.env.ACTIVATE_CACHE):
+            # IDE ids have to stay consistent with previous implementations.
+            # But add the macron name in the string generating the hash for non IDE elements, so
+            # that there using the same source file for different kind of macros won't generate
+            # equivalent hashes (ease the way for the users, _and_ more consistent with previous
+            # behaviors... Even if that is probably a breaking change).
+            less_ID_needs = "" if is_ide else ':'+self.MACRO_NAME
+
+            runner_path   = self.exo_py
+            if is_ide and not HashPathMode.is_legacy(self.env):
+                # Avoid any dependency on the domain/site name, if needed:
+                runner_path = Path(self.exo_py).relative_to(self.env.docs_dir_path)
+
+            to_hash = f"{ runner_path }{ self.IDE_VERT }{ less_ID_needs }"
+
+        else:
+            count   = self.env.macros_counters[ self.MACRO_NAME ].inc()
+            to_hash = f"{ self.env.page.url }-{ self.MACRO_NAME }-{ count }"
+                    # DO NOT lstrip, even if it looks ugly on the home page (nobody but me can see
+                    # it anyway... :rolleyes:), to keep implementation consistent for users...
+
+        hashed = self.id_to_hash(to_hash)
+        # print(hashed, to_hash)
+        return hashed
 
 
 
-    def id_to_hash(self, clear:str, no_id_path:str):
-        """ Hash the "clear version of it" to add as html id tail, prefix it, and check the
-            uniqueness of the hash across the whole website.
+    def id_to_hash(self, id_path:str, *_):
         """
+        Hash the "clear version of it" to add as html id tail, prefix it, and check the uniqueness
+        of the hash across the whole website.
 
-        hashed  = hashlib.sha1(clear.encode("utf-8")).hexdigest()
+        @*_: present for backward compatibility only.
+        """
+        to_hash = id_path if self.id is None else f"{ id_path }{ self.id }"
+        hashed  = hashlib.sha1(to_hash.encode("utf-8")).hexdigest()
         html_id = f"{ self.ID_PREFIX }{ hashed }"
 
-        if not self.env.is_unique_then_register(html_id, no_id_path, self.id):
-            raise PyodideMacrosNonUniqueIdError(
-                "\nThe same html id got generated twice.\nIf you are trying to use the same "
-                "set of files for different macros calls, use their ID argument (int >= 0) "
-                "to disambiguate them.\n\n"
-               f"Generated id: { html_id }\n"
-               f"ID values already in use: {self.env.get_registered_ids_for(no_id_path) }"
-               f"{ self }"
+        if not self.env.is_unique_then_register(html_id, id_path, self.id):
+            cache_option = self.env.ACTIVATE_CACHE and PLUGIN_CONFIG_SRC.get_plugin_path(
+                'build.activate_cache', no_deprecated=True
             )
+            msg = (
+                "The same html id got generated twice.\n"
+                "If you are trying to use the same set of files for different macros calls, use "
+                "their ID argument (int >= 0) to disambiguate them.\n\n"
+            )+ self.env.ACTIVATE_CACHE*(
+                "If you are encountering this problem with macros other than IDE or IDEv and you "
+                "do not need some extra speed rendering-wise, you also can deactivate the PMT "
+               f"cache, using `{ cache_option }: false`.\n\n"
+            )+(
+               f"ID values already in use:   { self.env.get_registered_ids_for(id_path) }\n"
+               f"Id generated with:          { to_hash }\n{ self.env.log() }"
+            )
+            raise PmtMacrosNonUniqueIdError(msg)
         return html_id
 
 
 
     def create_button(
         self,
-        btn_kind:    str,
+        btn_kind: str,
         *,
-        margin_left:    float = 0.2,
-        margin_right:   float = 0.2,
-        extra_btn_kls:  str   = "",
+        margin_left:   float = 0.2,
+        margin_right:  float = 0.2,
+        extra_btn_kls: str   = "",
+        tip_side:       P5BtnLocation = "",
         **kwargs
     ) -> str:
         """
@@ -459,6 +575,7 @@ class IdeManagerMdHtmlGenerator(IdeSectionsManager):
             margin_left   = margin_left,
             margin_right  = margin_right,
             extra_btn_kls = extra_btn_kls,
+            tip_side      = tip_side,
             **kwargs
         )
 
@@ -472,6 +589,7 @@ class IdeManagerMdHtmlGenerator(IdeSectionsManager):
         margin_left:    float = 0.2,
         margin_right:   float = 0.2,
         extra_btn_kls:  str   = "",
+        tip_side:       P5BtnLocation = "",
         **kwargs
     ) -> str:
         """
@@ -493,7 +611,7 @@ class IdeManagerMdHtmlGenerator(IdeSectionsManager):
         img = Html.img(src=img_link, kls=HtmlClass.skip_light_box, **img_style)
 
         tip: Tip = getattr(env.lang, lang_prop)
-        tip_span = Html.tooltip(tip, tip.em)
+        tip_span = Html.tooltip(tip, tip.em, tip_side=tip_side or P5BtnLocation.bottom)
 
         btn_style = f"margin-left:{margin_left}em; margin-right:{ margin_right }em;"
         if 'style' in kwargs:
@@ -517,7 +635,7 @@ def get_button_fields_data(btn_kind:str):
     Return the various property names to use for each kind of element (tooltip, image, ...),
     for the given initial button_name.
 
-    @returns:   png_name, lang_prop, js_method, color
+    @returns:   png_name, lang_prop, color
     """
     if btn_kind in BTNS_KINDS_CONFIG:
         return BTNS_KINDS_CONFIG[btn_kind]
@@ -578,19 +696,22 @@ class IdeManagerExporter(IdeManagerMdHtmlGenerator, metaclass=ABCMeta):
             self.editor_name, MacroPyConfig(**registered)
         )
 
-        name, is_tester = macro_name_to_src_and_is_tester(self.MACRO_NAME)
+        macro_data_name, in_macros_data = PmtPyMacrosName.get_macro_data_config_for(self.MACRO_NAME)
 
-        # IDE_tester macro is not exported, so exit directly:
-        if is_tester: return
+        # IDE_tester or playground macro is not exported, so exit directly:
+        if not in_macros_data:
+            return
 
-        if not self.env.all_macros_data:
+        if not self.env.all_macros_data:            # Defensive programming
             if self.env.is_dirty: return
-            raise BuildError("No MacroData instance registered yet! Seeking for "+self.MACRO_NAME)
+            raise PmtInternalError(
+                "No MacroData instance registered yet! Seeking for "+self.MACRO_NAME
+            )
 
         macro_data = self.env.all_macros_data[-1]
-        if macro_data.macro != name:
-            raise BuildError(
-                f"Wrong MacroData object: Expected {name} but was {macro_data.macro}"
+        if macro_data.macro != macro_data_name:     # Defensive programming
+            raise PmtInternalError(
+                f"Wrong MacroData object: Expected {macro_data_name} but was {macro_data.macro}"
             )
         macro_data.build_ide_manager_related_data(self)
 
@@ -610,11 +731,11 @@ class IdeManagerExporter(IdeManagerMdHtmlGenerator, metaclass=ABCMeta):
             ('python_libs',      [ p.name for p in map(Path,self.env.python_libs) ]),
             ('pypi_white',       self.env.limit_pypi_install_to),
             ('seq_run',          self.env.sequential_run),
-
             *zip(
                 ('run_group', 'order_in_group'),
                 self.get_page_group_and_order()
             ),
+            ("remove_assertions_stacktrace", self.env.remove_assertions_stacktrace),
         ]
         # All data related to files (python, REMs):
         yield from self.files_data.get_sections_data( with_corr=self.KEEP_CORR_ON_EXPORT_TO_JS )
@@ -637,23 +758,19 @@ class IdeManagerExporter(IdeManagerMdHtmlGenerator, metaclass=ABCMeta):
         The actual encoding of these states are left to determine by the current PageConfiguration:
             self.env.current_page_config.get_run_group_data(...) -> [int, int]
         """
-        group_id = self.run_group and self.run_group.strip('*')
-        is_skip  = group_id == 'SKIP'
+        group_id     = self.run_group and self.run_group.strip('*')
+        is_skip      = group_id == RUN_GROUP_SKIP
         has_priority = group_id and (self.run_group.startswith('*') or self.run_group.endswith('*'))
 
-        if isinstance(group_id,str) and is_skip and has_priority:
-            raise BuildError(
+        if isinstance(group_id, str) and is_skip and has_priority:
+            raise PmtMacrosInvalidArgumentError(
                 f"Invalid RUN_GROUP={self.run_group!r} argument: a skipped element cannot get priority."
-                f"{ self }"
+                f"{ self.env.log() }"
             )
 
-        in_sequential_run = (
-            self.env.sequential_run
-            and SequentialFilter.is_allowed(self, self.env)
-            and not is_skip
-        )
-        run_config =  self.env.current_page_config.get_run_group_data(
-            bool(in_sequential_run), bool(has_priority), group_id
+        in_sequential_run = not is_skip and SequentialFilter.is_allowed(self, self.env)
+        run_config = self.env.current_page_config.get_run_group_data(
+            in_sequential_run, bool(has_priority), group_id
         )
 
         return run_config
@@ -666,8 +783,7 @@ class IdeManagerExporter(IdeManagerMdHtmlGenerator, metaclass=ABCMeta):
         Guess an explicative enough py_name (when downloading the IDE content)
         """
         root_name = Path(self.env.page.url).stem
-        py_path   = Path(self.py_name).stem
-        py_name   = f"{root_name}-{py_path}".strip('-') or 'unknown'
+        py_name   = f"{ root_name }-{ self.py_name }".strip('-') or 'unknown'
         return py_name + '.py'
 
 
@@ -687,7 +803,7 @@ class IdeManagerExporter(IdeManagerMdHtmlGenerator, metaclass=ABCMeta):
         exclusions = excluded, excluded_methods, excluded_kws = [
             self._get_exclusions_prefixed_with(lst, pattern, slice)
             for lst,pattern,slice in (
-                (all_excluded,   '', 0),
+                (all_excluded,   r'(?!\d)\w+', 0),
                 (all_excluded,   r'[.](?!\d)\w+', 1),
                 (kws_candidates, '.+', 0),
             )
@@ -696,24 +812,25 @@ class IdeManagerExporter(IdeManagerMdHtmlGenerator, metaclass=ABCMeta):
 
         if excluded_kws and any(kw for kw in excluded_kws if kw not in PYTHON_KEYWORDS):
             wrongs=', '.join(sorted(repr(kw) for kw in excluded_kws if kw not in PYTHON_KEYWORDS))
-            raise BuildError(
-                f"Invalid python keywords for the `SANS` argument: { wrongs }.{ self }"
+            raise PmtMacrosInvalidArgumentError(
+                f"Invalid python keywords for the `SANS` argument: { wrongs }.{ self.env.log() }"
             )
 
         if 'globals' in excluded:
-            raise BuildError(
+            raise PmtMacrosInvalidArgumentError(
                 "It's not possible to use `SANS='globals`, because it would break pyodide "
-               f"itself.{ self }"
+               f"itself.{ self.env.log() }"
             )
 
+        # Check nothing missing (would mean invalid syntax):
         all_done_check = all_excluded + kws_candidates
         if all_done_check and len(all_done_check) != sum(map(len, exclusions)):
             all_done_check = { arg.lstrip('.') for arg in all_done_check }
             for lst in exclusions:
                 all_done_check.difference_update(lst)
             wrongs = ', '.join( map(repr, sorted(all_done_check)) )
-            raise BuildError(
-                f"Invalid `SANS` argument, containing: { wrongs }{ self }"
+            raise PmtMacrosInvalidArgumentError(
+                f"Invalid `SANS` argument, containing: { wrongs }{ self.env.log() }"
             )
 
         return excluded, excluded_methods, excluded_kws, white_list
@@ -732,10 +849,10 @@ class IdeManagerExporter(IdeManagerMdHtmlGenerator, metaclass=ABCMeta):
         return lst
 
 
-    def _get_exclusions_prefixed_with(self, exclusions_lst:List[str], pattern:str='', slice_on:int=None):
+    def _get_exclusions_prefixed_with(self, exclusions_lst:List[str], pattern:str, slice_on:int=None):
         if not exclusions_lst:
             return exclusions_lst
-        reg = re.compile( pattern or r'(?!\d)\w+' )
+        reg = re.compile(pattern)
         return [
             kw[slice_on:] if slice_on else kw
             for kw in exclusions_lst if reg.fullmatch(kw)
