@@ -20,7 +20,7 @@ If not, see <https://www.gnu.org/licenses/>.
 # pylint: disable=unused-argument
 
 
-from typing import ClassVar, Tuple, Union
+from typing import ClassVar
 from dataclasses import dataclass
 
 from pyodide_mkdocs_theme.pyodide_macros.html_dependencies.deps_class import DepKind
@@ -48,4 +48,9 @@ class AutoRun(PyBtn):
     DEPS_KIND: ClassVar[DepKind] = DepKind.run_macro
 
     def make_element(self) -> str:
+        """
+        {{ run(...) }} elements cannot be inserted in the markdown content now, otherwise they
+        will break the md rendering, adding new lines/paragraphs around the empty span. So the
+        html code is actually inserted through the plugin's on_page_context hook.
+        """
         return ""
