@@ -26,7 +26,8 @@ from mkdocs.exceptions import BuildError
 
 
 from ..pyodide_logger import logger
-from ..tools_and_constants import MACROS_WITH_INDENTS, HtmlClass
+from ..tools_and_constants import MACROS_WITH_INDENTS, HtmlClass, IdeConstants
+from ..exceptions import PmtMacrosInvalidArgumentError
 from ..html_dependencies.deps_class import DepKind
 from ..plugin.maestro_macros import MaestroMacros
 
@@ -119,11 +120,11 @@ def multi_qcm(env:MaestroMacros):
                 # If using default, pick plural when appropriate:
                 qcm_title = env.lang.qcm_title.one_or_many( len(questions_data) > 1 )
 
-                # But forbid title when the admonition will be removed later (JS)
             elif not admo_kind:
-                raise BuildError(
-                    "A multi_qcm with a `qcm_title` argument cannot use `admo_kind=None`: "
-                    "the title will be lost."
+                # But forbid title when the admonition will be removed later (JS)
+                raise PmtMacrosInvalidArgumentError(
+                    "A multi_qcm with a `qcm_title` argument cannot use `admo_kind=None` "
+                    "or the title would be lost."
                 )
 
             admo_qcm    = f'{ admo_kind or "!!!" } { admo_class } { xtra_classes } "{ qcm_title }"'
@@ -138,11 +139,11 @@ def multi_qcm(env:MaestroMacros):
                 auto_indent( f'\n{ FIX_P }<{ tag_list_of_qs } class="{ ol_ul_class }">\n' ),
             ]
 
-
         def qcm_close():
             admonition_lst.append(auto_indent(f"{ FIX_P }</{ tag_list_of_qs }>"))
 
 
+
         def question_open(question:str, n:int, lst_answers:List[int], default_multi, shuffled):
             is_multi  = len(lst_answers) > 1
             multi_kls = HtmlClass.qcm_multi if is_multi or default_multi else HtmlClass.qcm_single
@@ -153,14 +154,10 @@ def multi_qcm(env:MaestroMacros):
             admonition_lst.append(auto_indent(tag_open))
 
 
-        def question_close():
-            admonition_lst.append(auto_indent("</li>\n"))
-            # Extra linefeed for presentational purpose only
-
-
         def question_options(items):
-            """ Always use "md_in_html" approach, to simplify the construction. It is required
-                anyway when the first item starts with a code block...
+            """
+            Always use "md_in_html" approach, to simplify the construction. It is required anyway
+            when the first item starts with a code block...
             """
             admonition_lst.append(
                 auto_indent(f'{ FIX_P }<ul class="{ HtmlClass.py_mk_item_qcm }">')
@@ -172,6 +169,34 @@ def multi_qcm(env:MaestroMacros):
                 auto_indent("</ul>")
             )
 
+        def add_question_comment(comment:str):
+            encrypt_token = ()
+            if env.encrypt_comments:
+                env.set_current_page_insertion_needs(DepKind.qcm_encrypt)
+                encrypt_token = ( indent_comment(IdeConstants.encryption_token)+"\n", )
+
+            classes = ' '.join((
+                HtmlClass.py_mk_hidden,
+                HtmlClass.py_mk_comment_qcm,
+                HtmlClass.py_mk_encrypted_qcm * env.encrypt_comments,
+            ))
+            admonition_lst.extend((
+                auto_indent(f'??? tip { classes } "Remarque"')+"\n",
+                *encrypt_token,
+                indent_comment(comment)+"\n",
+                *encrypt_token,
+            ))
+
+            # Mermaid graphs inside REMs won't be seen, so make sure the script is added when
+            # the code blocks are found:
+            if "```mermaid" in comment:
+                env.set_current_page_insertion_needs(DepKind.mermaid)
+
+
+        def question_close():
+            admonition_lst.append(auto_indent("</li>\n"))
+            # Extra linefeed for presentational purpose only
+
 
 
         def validate_question_config(n_question, question, items, lst_correct, multi):
@@ -187,7 +212,7 @@ def multi_qcm(env:MaestroMacros):
             elif unknown:
                 msg = f"Correct answers contain invalid items number: {unknown}"
                 if 0 in unknown:
-                    msg += "\n(Note: correct answers are not given as indices and start with 1)"
+                    msg += "\n(Note: correct answers are not indices and start at 1, not 0!)"
 
             elif not lst_correct and (not multi or require_ans):
                 msg = ''.join((
@@ -207,10 +232,17 @@ def multi_qcm(env:MaestroMacros):
                     f"\n{ question !r}"
                     f"\n\n{msg}"
                 )
-                if env._dev_mode:                                   # pylint: disable=protected-access
-                    logger.error('QCMs - EXPECTED (x2)\n'+msg)
-                else:
-                    raise BuildError(msg)
+                raise PmtMacrosInvalidArgumentError(msg)
+
+
+        def get_comment_and_options(a=None, b=None):
+            """
+            Extract appropriately the two last optional values (question options configuration
+            and question commentary).
+            """
+            dct     = a if isinstance(a, dict) else b if isinstance(b, dict) else {}
+            comment = a if isinstance(a, str) else b if isinstance(b, str) else ""
+            return comment, dct
 
 
         #------------------------------------------------------------------
@@ -225,17 +257,18 @@ def multi_qcm(env:MaestroMacros):
                 qcm_format(q),
                 [*map(qcm_format,items)],
                 sorted(ans),
-                dct[0] if dct else {}
+                *get_comment_and_options(*extras),
             ]
-            for q, items, ans, *dct in questions
+            for q, items, ans, *extras in questions
         ]
 
         tag_list_of_qs = tag_list_of_qs or ('ol' if len(questions_data) > 1 else 'ul')
         indent         = env.get_macro_indent()
         auto_indent    = auto_indenter_factory(indent)
+        indent_comment = auto_indenter_factory(indent + '    ')
         admonition_lst = qcm_start()
 
-        for n, (question, items, lst_answers, extra_dct) in enumerate(questions_data, 1):
+        for n, (question, items, lst_answers, comment, extra_dct) in enumerate(questions_data, 1):
 
             shuffled = extra_dct.get('shuffle', shuffle or shuffle_items)
             is_multi = extra_dct.get('multi',   multi)
@@ -243,6 +276,8 @@ def multi_qcm(env:MaestroMacros):
 
             question_open(question, n, lst_answers, is_multi, shuffled)
             question_options(items)
+            if comment:
+                add_question_comment(comment)
             question_close()
 
         qcm_close()
@@ -271,14 +306,14 @@ def auto_indenter_factory(indent:str):
 
     indent += '    '
 
-    def indenter(content:str, wrap_li=False, lvl:int=1):
+    def indenter(content:str, wrap_li=False):
         """
         Takes a @content, possibly multiline, and indent all lines (including the first) with the
         base @indent and an extra @lvl, where each level counts for 4 space characters.
 
         If @wrap_li is True, handle the "li" element appropriately, meaning:
             - prepend with "* " for simple content
-            - surround the item with `<li markdown>...</li> if it's a code bloc`
+            - surround the item with `<li markdown>...</li>
         """
         if wrap_li:
             content = f'<li class="{ HtmlClass.py_mk_item_qcm }" markdown="1">\n{ content }\n</li>'
