@@ -26,7 +26,9 @@ from functools import lru_cache
 
 from mkdocs.exceptions import BuildError
 
+
 from .tools_and_constants import LZW_DELIMITER, DebugConfig
+from .exceptions import PmtEncryptionError, PmtInternalError, PmtCodeFenceTitleQuotesError
 
 if TYPE_CHECKING:
     from .plugin import PyodideMacrosPlugin
@@ -102,12 +104,15 @@ def items_comma_joiner(lst:List[str], join:str='et'):
 
 
 
-def add_indent(content:str, indent:str):
+def add_indent(content:str, indent:str, leading=False):
     """
-    Add the given indentation to all lines containing non space characters.
+    Add the given indentation to all lines containing non space characters (hence, leave
+    empty lines as they are: this is compliant with the md rendering engine).
+
+    If @leading is True, also add the indent at the beginning.
     """
     out = re.sub(r"(\n *)(?=\S)", lambda m:m[0]+indent, content)
-    return out
+    return leading * indent + out
 
 
 
@@ -117,7 +122,8 @@ def build_code_fence(
     indent:str="",
     line_nums=1,
     lang:str='python',
-    title:str=""
+    title:str="",
+    attrs:str="",
 ) -> str :
     """
     Build a markdown code fence for the given content and the given language.
@@ -131,19 +137,19 @@ def build_code_fence(
                      the given int value as starting line number.
     @lang (="python"): language to use to format the resulting code block.
     @title: title for the code block, if given. Note: the title cannot contain quotes `"`
+    @attrs: md attributes to add to the code block (`".class-x .inline .end"`)
     """
     line_nums = f'linenums="{ line_nums }"' if line_nums else ""
     if title:
         if '"' in title:
-            raise BuildError(
+            raise PmtCodeFenceTitleQuotesError(
                 f'Cannot create a code fence template with a title containing quotes:\n'
                 f"  {lang=}, {title=!r}\n{content}"
             )
         title = f'title="{ title }"'
-
     lst = [
         '',
-        f"```{ lang } { title } { line_nums }",
+        f"```{ lang } {'{'} { title } { line_nums } { attrs } {'}'}",
         *content.strip('\n').splitlines(),
         "```",
         '',
@@ -201,7 +207,7 @@ def compress_LZW(txt:str, env:'PyodideMacrosPlugin'):
 
     tome  = set(txt) - set(NO_HTML)                  # remove tags tokens
     if LZW_DELIMITER in tome:
-        raise BuildError(
+        raise PmtEncryptionError(
             "Cannot encrypt data because the text already contains the delimiter used to "
             "identify sections in the encoded content. Solutions to this problem are:\n"
            f"    1. Don't use {LZW_DELIMITER!r} in the content.\n"
@@ -274,7 +280,7 @@ def _check_decode_LZW(txt, dct, size, output_with_table):
             min(len(txt), len(decoded))
         )
         # (Path.cwd() / "encoded").write_text(output_with_table.replace('\x1e', '\n'), encoding='utf-8')
-        raise BuildError(f'''
+        raise PmtInternalError(f'''
 Failed to decode...
 
 Alpha: ]]{ alpha }[[ (len={len(alpha)})
