@@ -22,11 +22,12 @@ If not, see <https://www.gnu.org/licenses/>.
 from typing import List
 from mkdocs.config import config_options as C
 
+from pyodide_mkdocs_theme.pyodide_macros.plugin.config._string_tools_and_constants import InclusionProfile
 
-from pyodide_mkdocs_theme.pyodide_macros.plugin.tools.test_cases import CASES_OPTIONS
-from pyodide_mkdocs_theme.pyodide_macros.tools_and_constants import PmtTests, SequentialFilter, SequentialRun
+
+from ....tools_and_constants import DecreaseAttemptsMode, DeprecationLevel, HashPathMode, PageInclusion, SequentialFilter, SequentialRun, NamedTestCase
 from ..common_tree_src import CommonTreeSrc
-from ..config_option_src import ConfigOptionSrc
+from ..config_option_src import ConfigOptionDeprecated, ConfigOptionSrc
 from ..sub_config_src import SubConfigSrc
 from .docs_dirs_config import DOCS_CONFIG, to_page
 from ...tools.options_alterations import sanitize_decrease_attempts_on_user_code_failure
@@ -61,55 +62,98 @@ CommonTreeSrc.DEFAULT_DOCS_URL_TEMPLATE = to_page(DOCS_CONFIG) / '#{py_macros_pa
 
 BUILD_CONFIG = SubConfigSrc(
     'build',
-    docs = """
+    extra_docs = """
     Réglages concernant la construction de la documentation ou qui impactent la façon
     dont le contenu des pages est construit.
     """,
     elements = (
 
+    # KEEP THIS ONE: used for testing purposes.
+    ConfigOptionDeprecated(
+        'activate_cache', bool, default=None, # default=True
+        extra_docs = """
+            Active ou non le cache permettant de stocker les données relatives aux codes python et
+            REMs des différentes macros (IDE, terminal, ...).
+
+            Si `True` :
+
+            * Rend `mkdocs serve` plus rapide après le premier rendu
+            * Il est plus souvent nécessaire d'utiliser les arguments `ID` des différentes macros.
+
+            Si `False`:
+
+            * Les arguments ID ne sont jamais nécessaires pour les macros autres que les IDEs.
+            * Les contenus sont recalculés systématiquement à chaque rendu, à partir des données
+            des fichiers individuels
+        """,
+        yaml_desc="""
+            Activate or not the macros cache. If activated, the ID argument is more often necessary.
+        """,
+    ),
+
     ConfigOptionSrc(
-        'deprecation_level', str, conf_type=C.Choice( ('error', 'warn'), default='error'),
-        docs = """
+        'deprecation_level', str, conf_type=C.Choice( DeprecationLevel.VALUES, default=DeprecationLevel.error),
+        extra_docs = """
             Comportement utilisé lors d'un build/serve lorsqu'une option obsolète est utilisée.
+
+            _ATTENTION:_ concernant les fichiers {{ meta() }}, seule la valeur sélectionnée depuis
+            le fichier `mkdocs.yml` est prise en compte. Les valeurs configurées depuis ces fichiers
+            n'impactent que les validations des données dans les entêtes des pages markdown.
         """,
         yaml_desc="Behavior when a deprecated feature is used."
     ),
 
     ConfigOptionSrc(
         'encrypted_js_data', bool, default=True,
-        docs="""
+        extra_docs="""
             Si `True`, les données de configuration des IDEs, terminaux et py_btns sont encodées.
+
             Si des problèmes de décompression des données sont rencontrés, cette option peut être
-            désactivée.
+            désactivée, mais cela implique que toutes les données des codes python (notamment les
+            contenus des sections `secrets` ou `corr`) seront accessibles à quelqu'un fouillant
+            explorant le DOM de la page via l'inspecteur du navigateur.
         """,
         yaml_desc="Compress or not configuration data of IDEs, terminals, ...",
-        # yaml_desc="Compression ou non des données de configuration des IDEs, terminaux, ...",
-        # """
-        # If True, the configuration data for IDEs, terminals, py_btns, ... are encrypted.
-        # In case of decompression troubles, you may want to deactivate this option.
-        #
-        # Note that the ides.encrypt_alpha_mode setting also applies to these encryptions.
-        # """
+    ),
+
+    ConfigOptionSrc(
+        'extra_pyodide_sections', List[str], conf_type=C.ListOfItems(C.Type(str), default=[]),
+        extra_docs="""
+            Liste de chaînes de caractères additionnelles autorisées en tant que nom de sections
+            pour les entêtes `PYODIDE:{section}` des fichiers python du thème.
+        """,
+        yaml_desc="""
+            Extra `PYODIDE:{section}` names authorized in the python files.
+        """,
     ),
 
     ConfigOptionSrc(
         'forbid_macros_override', bool, default=True,
-        docs="""
-            Si `True`, `PyodideMacrosError` est levée lorsque deux macros du même nom sont
+        extra_docs="""
+            Si `True`, `PmtMacrosError` est levée lorsque deux macros du même nom sont
             enregistrées par le plugin.
         """,
         yaml_desc="""
             If `True` registering different macros with the same name will raise a
-            `PyodideMacrosError`.
+            `PmtMacrosError`.
         """,
-        # """
-        # If True, `PyodideMacrosError` is raised when two macros are registered with the same name.
-        # """
+    ),
+
+    ConfigOptionSrc(
+        'ides_id_hash_mode', str, conf_type=C.Choice( HashPathMode.VALUES, default=HashPathMode.legacy),
+        extra_docs = f"""
+            Définit le mode de construction des id html des IDEs. Changer de mode rend toutes les
+            données utilisateurs stockées en localeStorage invalide !
+
+            Si le mode {HashPathMode.relative!r} est utilisé, un changement de nom de domaine ou de
+            la valeur de `site_url` n'affecte pas les valeurs des identifiant html des IDEs.
+        """,
+        yaml_desc="Behavior used to compute the IDEs' html ids."
     ),
 
     ConfigOptionSrc(
         'ignore_macros_plugin_diffs', bool, default=False,
-        docs = """
+        extra_docs = """
             Passer à `#!py True` pour éviter la vérification de compatibilité de la
             configuration du plugin `PyodideMacroPlugin` avec celle du plugin original
             des macros, `MacrosPlugin`.
@@ -139,34 +183,11 @@ BUILD_CONFIG = SubConfigSrc(
             Deactivate the compatibility check of PyodideMacrosPlugin configuration against the
             MacrosPlugin one.
         """,
-        # yaml_desc="""
-        #     Désactive la vérification de compatibilité entre PyodideMacrosPlugin et MacrosPlugin
-        # """,
-        # """
-        # Set to True to bypass the compatibility check of the `PyodideMacrosPlugin` config against
-        # the original `MacrosPlugin` one.
-
-        # ??? note "Reasons behind this verification"
-
-        #     `MacrosPlugin` is configured the "old fashion" way while `PyodideMacrosPlugin` is
-        #     using mkdocs 1.5+ Config classes. This means that the `MacrosPlugin` configuration
-        #     had to be hardcoded in the `PyodideMacrosPlugin` config.
-
-        #     Because of this, any config change on the `MacrosPlugin` side could break
-        #     `PyodideMacrosPlugin` without warning, so this verification enforces the expected
-        #     implementation of the parent class.
-
-        #     If ever something unexpected is found, the build will be aborted, because no
-        #     guarantees can be given about the correctness of the build in such situation.
-
-        #     In case of minor changes, this option will allow the build, but use it "at your own
-        #     risks".
-        # """
     ),
 
     ConfigOptionSrc(
         'load_yaml_encoding', str, default='utf-8',
-        docs="""
+        extra_docs="""
             Encodage à utiliser lors du chargement de données YAML avec les fonctionnalités
             originales de MacrosPlugin :
 
@@ -177,21 +198,11 @@ BUILD_CONFIG = SubConfigSrc(
         yaml_desc="""
             Encoding used when loading yaml files within the original macros plugin.
         """,
-        # yaml_desc="""
-        #     Encodage utilisé pour charger des fichiers yaml depuis le plugin des macros.
-        # """,
-        # """
-        # Encoding to use when loading yaml data with the original MacrosPlugin functionalities :
-
-        # The original method doesn't use any encoding argument, which can lead to different
-        # behaviors between Windows and Linux (typically: during a pipeline vs working locally
-        # with Windows).
-        # """
     ),
 
     ConfigOptionSrc(
         'macros_with_indents', List[str], conf_type=C.ListOfItems(C.Type(str), default=[]),
-        docs="""
+        extra_docs="""
             Permet d'enregistrer des macros personnalisées (liste de chaînes de caractères), qui
             insèrent du contenu markdown multilignes, pour pouvoir indenter  correctement le
             contenu dans la page :
@@ -204,22 +215,11 @@ BUILD_CONFIG = SubConfigSrc(
             List of macros inserting multiline contents in the pages (allowing to use
             `plugin.indent_macro(markdown)` from them).
         """,
-        # yaml_desc="""
-        #     Liste de macros insérant des contenus multilignes (permet d'y utiliser
-        #     `PyodideMacrosPlugin.indent_macro`).
-        # """,
-        # """
-        # Allow to register external macros, as a list of strings, that will need to insert properly
-        # indented multiline contents in the page.
-        # Once a macro is registered in this list, it can call `env.get_macro_indent()` at runtime
-        # to get the indentation level (as a string of spaces) of the macro call in the markdown
-        # source file.
-        # """
     ),
 
     ConfigOptionSrc(
         'meta_yaml_allow_extras', bool, default=False,
-        docs="""
+        extra_docs="""
             Définit s'il est possible d'ajouter dans les fichiers {{meta()}} des données autres
             que celles relatives au plugin lui-même.
 
@@ -229,19 +229,23 @@ BUILD_CONFIG = SubConfigSrc(
             la valeur est à `#!yaml true`, il est alors possible d'ajouter d'autres variables,
             mais les fautes de frappes dans les premiers niveaux ne peuvent plus être identifiées
             (exemple : `temrs.cut_feedback` au lieu de `terms.cut_feedback`).
+
+            _ATTENTION:_ concernant les fichiers {{ meta() }}, seule la valeur sélectionnée depuis
+            le fichier `mkdocs.yml` est prise en compte. Les valeurs configurées depuis ces fichiers
+            n'impactent que les validations des données dans les entêtes des pages markdown.
         """,
         yaml_desc="Allow definition of extra variables/options in the `.meta.pmt.yml` files.",
     ),
 
     ConfigOptionSrc(
         'meta_yaml_encoding', str, default='utf-8',
-        docs="Encodage utilisé pour charger les [fichiers `.meta.pmt.yml`](--custom/metadata/).",
+        extra_docs="Encodage utilisé pour charger les [fichiers `.meta.pmt.yml`](--custom/metadata/).",
         yaml_desc="Encoding to use when loading `.meta.pmt.yml` files."
     ),
 
     ConfigOptionSrc(
         'python_libs', List[str], conf_type=C.ListOfItems(C.Type(str), default=['py_libs']),
-        docs="""
+        extra_docs="""
             Liste de répertoires de [bibliothèques python](--custom-libs) qui doivent être
             importables dans Pyodide.
 
@@ -256,32 +260,22 @@ BUILD_CONFIG = SubConfigSrc(
         yaml_desc="""
             List of custom python packages to make available at runtime in Pyodide environment.
         """,
-        # yaml_desc="""
-        #     Liste des bibliothèques personnalisée à rendre disponibles sur le site final.
-        # """,
-        # """
-        # List of directories of python packages that must be importable in Pyodide.
-
-        # An error is raised if:
-
-        # * The given name isn't an existing directory (unless it's the default value, `py_libs`).
-        # * The directory isn't at project root level.
-        # * The directory isn't a python package (aka: it doesn't have an `__init__.py` file).
-        # """
     ),
 
     ConfigOptionSrc(
         'limit_pypi_install_to', List[str], is_optional=True, conf_type=C.ListOfItems(C.Type(str)),
-        docs="""
-            Si cette liste est définie, seules les imports dont le nom de bibliothèque
-            figure dans cette liste seront autorisés à déclencher une installation
-            automatique depuis PyPI. Noter que :
+        extra_docs="""
+        Si cette liste est définie, seules les imports dont le nom de bibliothèque
+        figure dans cette liste seront autorisés à déclencher une installation
+        automatique depuis PyPI. Noter que :
 
-            * C'est le nom de l'import dans le code python qui doit être renseigné
-              (ex : `PIL` pour interdire l'installation de `pillow`).
-            * Utiliser `[]` interdit toutes les installations automatiques depuis PyPI.
-            * Mettre cette option à `null` (valeur par défaut) autorise toutes les
-              requêtes vers PyPI.
+        * C'est le nom de l'import dans le code python qui doit être renseigné
+        (ex : `PIL` pour interdire l'installation de `pillow`).
+
+        * Utiliser `[]` interdit toutes les installations automatiques depuis PyPI.
+
+        * Mettre cette option à `null` (valeur par défaut) autorise toutes les
+        requêtes vers PyPI.
         """,
         yaml_desc="""
             If defined, only the package names in this list are allowed to be automatically
@@ -289,9 +283,20 @@ BUILD_CONFIG = SubConfigSrc(
         """,
     ),
 
+    ConfigOptionDeprecated(
+        'show_cache_refresh', bool, default=None, # default=False
+        extra_docs="""
+            Si `#!yaml true`, des messages sont ajoutés dans la console permettant de voir quand
+            les caches des fichiers du thèmes sont mis à jour (debugging purpose...).
+        """,
+        yaml_desc="""
+            Show additional messages in the console about internal caches updates.
+        """,
+    ),
+
     ConfigOptionSrc(
         'skip_py_md_paths_names_validation', bool, default=False,
-        docs = """
+        extra_docs = """
             Par défaut, les noms de chemin de tous les fichiers `.py` et `.md` présents dans
             le `docs_dir` sont vérifiés pour s'assurer qu'ils ne contiennent aucun caractère
             autre que des lettres, des chiffres, des points ou des tirets. Cela garantit le
@@ -303,23 +308,14 @@ BUILD_CONFIG = SubConfigSrc(
             ... À Utiliser  à vos risques et périls.
         """,
         yaml_desc="Deactivate the sanity check of the directories and files in the `docs_dir`.",
-        # yaml_desc="Désactive le contrôle des nom de dossiers et fichiers.",
-        # """
-        # By default, the path names of all the `.py` and `.md` files present in the docs_dir are
-        # checked so that they do not contain any character other than letters, digits, dots or
-        # dashes. This ensures the macros related to IDEs will work properly.
-
-        # If unwanted characters are found, a BuildError is raised, but this verification can be
-        # turned off by setting this flag to True. Use it at your own risks.
-        # """
     ),
 
     ConfigOptionSrc(
         'tab_to_spaces',int, default=-1,
-        docs="""
+        extra_docs="""
             Si cette option est définie avec une valeur positive (ou nulle), les tabulations
             trouvées avant un appel à une macro multiligne (voir l'option
-            [`macros_with_indent`](--pyodide_macros_build_macros_with_indents)) seront
+            [`macros_with_indenst`](--pyodide_macros_build_macros_with_indents)) seront
             automatiquement converties en utilisant ce nombre d'espaces.
 
             __Aucune garantie n'est alors donnée quant à la correction du résultat__.
@@ -342,26 +338,12 @@ BUILD_CONFIG = SubConfigSrc(
             Number os space characters used to replace tabulations on the left of multiline
             macro calls.
         """,
-        # yaml_desc="""
-        #     Nombre d'espaces pour remplacer les tabulation lors de la gestion des macros avec
-        #     indentation.
-        # """,
-        # """
-        # If set to a positive value (or 0), tabs characters found in front of a multiline macro
-        # call will automatically be converted using this number of spaces (see
-        # [`macros_with_indent`]
-        # (--pyodide_macros_build_macros_with_indents)
-        # option).
-        # <br>_There are NO guarantees about the correctness of the result_.
-
-        # If a conversion is done, a warning will be shown in the console to find and modify more
-        # easily the problematic macros calls.
-        # """
     ),
 
     ConfigOptionSrc(
-        '_pmt_meta_filename', str, default=".meta.pmt.yml", in_yaml_docs=False,
-        docs = "Nom des fichiers de configuration des métadonnées pour le thème.",
+        '_pmt_meta_filename', str, default=".meta.pmt.yml",
+        inclusion_profile = InclusionProfile.config,
+        extra_docs = "Nom des fichiers de configuration des métadonnées pour le thème.",
         yaml_desc="Name used for the Pyodide-MkDoc-Theme meta files."
     ),
 
@@ -384,7 +366,7 @@ BUILD_CONFIG = SubConfigSrc(
     #     moved_to = 'ides.forbid_corr_and_REMs_with_infinite_attempts',
     # ),
     # ConfigOptionDeprecated(
-    #     'bypass_indent_errors', bool, dep_status=DeprecationStatus.removed,
+    #     'bypass_indent_errors', bool, deprecation_status=DeprecationStatus.removed,
     # ),
 ))
 
@@ -402,14 +384,14 @@ BUILD_CONFIG = SubConfigSrc(
 
 IDES_CONFIG = SubConfigSrc(
     'ides',
-    docs = """
+    extra_docs = """
     Réglages spécifiques aux IDEs (comportements impactant l'utilisateur et les exécutions).
     """,
     elements = (
 
     ConfigOptionSrc(
         'deactivate_stdout_for_secrets', bool, default=True,
-        docs="""
+        extra_docs="""
             Détermine si la sortie standard (stdout) sera visible dans les terminaux lors
             des tests secrets ou non.
         """,
@@ -421,9 +403,10 @@ IDES_CONFIG = SubConfigSrc(
 
     ConfigOptionSrc(
         'decrease_attempts_on_user_code_failure', bool,
-        conf_type=C.Choice(('editor', 'public', 'secrets', True, False), default='editor'),
-        value_transfer_processor=sanitize_decrease_attempts_on_user_code_failure,
-        docs="""
+        conf_type=C.Choice(DecreaseAttemptsMode.VALUES+(True, False), default=DecreaseAttemptsMode.editor),
+        yaml_schema_dct = {"enum": [*DecreaseAttemptsMode.VALUES]},
+        value_transfer_processor = sanitize_decrease_attempts_on_user_code_failure,
+        extra_docs="""
             Les validations sont grossièrement constituées de 4 étapes, exécutant les éléments
             suivants :
 
@@ -456,23 +439,12 @@ IDES_CONFIG = SubConfigSrc(
         yaml_desc="""
             Define from which step an error will consume an attempt, during a validation.
         """,
-        # yaml_desc="""
-        #     Défini si les échecs avant la section `secrets` comptent déjà pour un essai ou non,
-        #     lors des validations.
-        # """,
-        # """
-        # If true, any failure when running the user code during a validation will decrease the
-        # number of attempts left. Note this means even syntax errors will decrease the count.
-
-        # When this option is set to False, any error raised within the code of the editor will stop
-        # the validation process without modifying the number of attempts left.
-        # """
     ),
 
     ConfigOptionSrc(
         'encrypt_alpha_mode', str,
         conf_type=C.Choice(('direct', 'shuffle', 'sort'), default='direct'),
-        docs="""
+        extra_docs="""
             Les contenus (codes, corrections & remarques) sont transmis de mkdocs aux pages html
             en utilisant des données compressées. L'encodage est réalisé avec l'algorithme LZW,
             et cette option contrôle la manière dont l'alphabet/la table initiale est construit à
@@ -487,24 +459,11 @@ IDES_CONFIG = SubConfigSrc(
             Define in what order the characters of the content are pushed in the LZW compression
             table (by default: `direct`, `shuffle`, `sort`).
         """,
-        # yaml_desc="""
-        #     Gestion de l'alphabet initial lors des compression LZW (`direct` par défaut, `shuffle`
-        #     ou `sort`).
-        # """,
-        # """
-        # Original contents are passed from mkdocs to the JS environment using compressed data. The
-        # encoding is done with LZW algorithm, and this option controls how the LZW initial alphabet
-        # is built from the content to encode:
-
-        # - `"direct"`: the alphabet is using all symbols in order.
-        # - `"shuffle"`: the alphabet is randomized.
-        # - `"sort"`: all symbols are sorted in natural order.
-        # """
     ),
 
     ConfigOptionSrc(
         'encrypt_corrections_and_rems', bool, default=True,
-        docs="""
+        extra_docs="""
             Si activé, le contenu de la div HTML de la correction et des remarques, sous
             les IDEs, sera compressé lors de la construction du site.
 
@@ -516,24 +475,11 @@ IDES_CONFIG = SubConfigSrc(
             Compress or not the solutions and remarks below IDEs (deactivate only for
             debugging purpose).
         """,
-        # yaml_desc="""
-        #     Compression ou non des corrections et remarques sous les IDEs (désactiver pour
-        #     debugging uniquement).
-        # """,
-        # """
-        # If True, the html div containing correction and remarks below IDEs will be encrypted at
-        # build time.
-
-        # Passing this to False can be useful during development, but the value should _ALWAYS_
-        # be True on the deployed website: keep in mind the search engine can otherwise make
-        # surface contents from solutions and remarks as suggestions when the user is using
-        # the search bar.
-        # """
     ),
 
     ConfigOptionSrc(
         'export_zip_prefix', str, default="",
-        docs="""
+        extra_docs="""
             Préfixe ajouté au début du nom des archives zip créées avec les contenus des éditeurs
             des IDEs configurés comme exportable (argument [`EXPORT=True`](--IDE-EXPORT)).
             Si `{{ config_validator('ides.export_zip_prefix',tail=1) }}` n'est pas une chaîne
@@ -548,7 +494,7 @@ IDES_CONFIG = SubConfigSrc(
 
     ConfigOptionSrc(
         'export_zip_with_names', bool, default=False,
-        docs="""
+        extra_docs="""
             Si `#!py True`, au moment où un utilisateur demandera de créer l'archive zip avec
             tous les codes des IDEs de la page [configurés pour être exportés](--IDE-EXPORT),
             une fenêtre s'ouvrira lui demandant d'indiquer son nom. Une fois le nom renseigné,
@@ -562,7 +508,7 @@ IDES_CONFIG = SubConfigSrc(
 
     ConfigOptionSrc(
         'forbid_corr_and_REMs_with_infinite_attempts', bool, default=True,
-        docs = f"""
+        extra_docs = f"""
             Lors de la construction des IDEs, si une section `corr`, un fichier `REM` ou `
             VIS_REM` existent et que le nombre de tentatives est illimité, ce contenu ne
             sera jamais accessible à l'utilisateur, sauf s'il réussit les tests.
@@ -570,19 +516,11 @@ IDES_CONFIG = SubConfigSrc(
             { FORBID_SHORT_CONFIG }
         """,
         yaml_desc = EN_FORBID_SHORT_CONFIG,
-        # """
-        # When building IDEs, if a `corr` section, a REM file or a VIS_REM file exist while the
-        # number of attempts is infinite, that content will never become accessible to the user,
-        # unless they pass the tests.
-
-        # By default, this situation is considered invalid and `BuildError` will be raised.
-        # If this is the desired behavior, set this option to false.
-        # """
     ),
 
     ConfigOptionSrc(
         'forbid_hidden_corr_and_REMs_without_secrets', bool, default=True,
-        docs=f"""
+        extra_docs=f"""
             Lors de la construction des IDEs, le bouton de validation n'apparaît que si une
             section `secrets` existe.
             <br>Si des sections `corr` ou des fichiers `REM` existent alors qu'aucune section
@@ -592,29 +530,31 @@ IDES_CONFIG = SubConfigSrc(
             { FORBID_SHORT_CONFIG }
         """,
         yaml_desc = EN_FORBID_SHORT_CONFIG,
-        # """
-        # When building IDEs, the validation button will appear only when a `secrets` section exist.
-        # If none is given while a corr section or REM files exist, their content will never be
-        # available to the user because of the lack of validation button in the interface.
-
-        # By default, this situation is considered invalid and `BuildError` will be raised.
-        # If this is the desired behavior, set this option to false.
-        # """
     ),
 
     ConfigOptionSrc(
         'forbid_secrets_without_corr_or_REMs', bool, default=True,
-        docs = FORBID_SHORT_CONFIG,
+        extra_docs = FORBID_SHORT_CONFIG,
         yaml_desc = EN_FORBID_SHORT_CONFIG,
-        # """
-        # By default, this situation is considered invalid and `BuildError` will be raised.
-        # If this is the desired behavior, set this option to false.
-        # """
+    ),
+
+    ConfigOptionSrc(
+        'remove_assertions_stacktrace', bool, default=False,
+        extra_docs="""
+            Si activé (`True`), les stacktrace des messages d'erreur des assertions sont supprimées
+            et seul le message d'erreur avec le type d'erreur est conservé (dernière ligne).
+
+            Ceci s'applique à n'importe quel moment des exécutions.
+        """,
+        yaml_desc="""
+            If True, the stacktrace of assertions messages will be removed (except the very last line/
+            part of the message).
+        """,
     ),
 
     ConfigOptionSrc(
         'show_only_assertion_errors_for_secrets', bool, default=False,
-        docs="""
+        extra_docs="""
             Si activé (`True`), la stacktrace des messages d'erreur sera supprimée et
             seuls les messages des assertions resteront inchangées lorsqu'une erreur
             sera levée pendant les tests secrets.
@@ -630,32 +570,27 @@ IDES_CONFIG = SubConfigSrc(
             }} |
         """,
         yaml_desc="""
-            If True, the stack trace of all error messages will be suppressed and only
+            If True, the stacktrace of all error messages will be suppressed and only
             assertion messages will be left unchanged, when an error is raised during the
             secret tests.
-        """
-        # yaml_desc="""
-        #     Réduit drastiquement les informations visibles dans les messages d'erreurs
-        #     (suppression stacktrace + pour les erreur autre que `AssertionError`, seulement
-        #     le type d'erreur est donné).
-        # """,
+        """,
     ),
 
     ConfigOptionSrc(
         'editor_font_family', str, default="monospace",
-        docs = "Police de caractère à utiliser pour les éditeurs des IDEs.",
+        extra_docs = "Police de caractère à utiliser pour les éditeurs des IDEs.",
         yaml_desc = "Font family used in IDEs' editor.",
     ),
 
     ConfigOptionSrc(
         'editor_font_size', int, default=15,
-        docs = "Taille de la police de caractère les éditeurs des IDEs.",
+        extra_docs = "Taille de la police de caractères pour les éditeurs des IDEs.",
         yaml_desc = "Font size used in IDEs' editor.",
     ),
 
     ConfigOptionSrc(
         'ace_style_dark', str, default="tomorrow_night_bright",
-        docs = """
+        extra_docs = """
             Thème de couleur utilisé pour les éditeurs des IDEs en mode sombre ([liste des
             thèmes disponibles][ace-themes]: utiliser le noms des fichiers `js` sans l'extension).{{
             pmt_note("Ce réglage est écrasé par l'ancienne façon de modifier le thème, en
@@ -666,7 +601,7 @@ IDES_CONFIG = SubConfigSrc(
 
     ConfigOptionSrc(
         'ace_style_light', str, default="crimson_editor",
-        docs = """
+        extra_docs = """
             Thème de couleur utilisé pour les éditeurs des IDEs en mode clair ([liste des thèmes
             disponibles][ace-themes]: utiliser le noms des fichiers `js` sans l'extension).{{
             pmt_note("Ce réglage est écrasé par l'ancienne façon de modifier le thème, en
@@ -704,12 +639,12 @@ IDES_CONFIG = SubConfigSrc(
 SEQUENTIAL_CONFIG = SubConfigSrc(
     'sequential',
     long_accessor = True,
-    docs = "Réglages Pour lier les exécutions de différents éléments entre elles.",
+    extra_docs = "Réglages Pour lier les exécutions de différents éléments entre elles.",
     elements = (
     ConfigOptionSrc(
         'run', str,
         conf_type = C.Choice(SequentialRun.VALUES, default=SequentialRun.none),
-        docs = """
+        extra_docs = """
             Cette option permet d'obtenir des executions liées pour différents éléments dans
             une page, un peu à la façon des Notebooks Jupyter, où l'on peut exécuter toutes
             les cellules en une fois.
@@ -731,7 +666,7 @@ SEQUENTIAL_CONFIG = SubConfigSrc(
     ConfigOptionSrc(
         'only', List[str],
         conf_type = C.ListOfItems(C.Choice(SequentialFilter.VALUES), default=list(SequentialFilter.VALUES)),
-        docs = """
+        extra_docs = """
             Cette option permet d'obtenir des executions liées pour différents éléments dans une
             page. Quand un utilisateur lance un élément dans la page (IDE, terminal, py_btn),
             s'il existe des éléments plus haut dans la page qui n'ont pas encore été exécutés,
@@ -773,21 +708,25 @@ SEQUENTIAL_CONFIG = SubConfigSrc(
 
 QCMS_CONFIG = SubConfigSrc(
     'qcms',
-    docs = "Réglages spécifiques aux QCMs.",
+    extra_docs = "Réglages spécifiques aux QCMs.",
     elements = (
     ConfigOptionSrc(
         'forbid_no_correct_answers_with_multi', bool, default=True,
-        docs="""
+        extra_docs="""
             Si désactivé (`False`), une question sans réponse correcte fournie, mais marquée comme
             `multi=True`, est considérée comme valide. Si cette option est réglée à `True`, cette
             situation lèvera une erreur.
         """,
         yaml_desc = "Allow to disambiguate MCQ and SCQ when needed.",
-        # yaml_desc="Permet de clarifier entre QCM et QCU quand ambiguë.",
-        # """
-        # If False, a question with no correct answer provided, but that is tagged as `multi=True`
-        # is considered valid. If this option is set to True, that situation will raise an error.
-        # """
+    ),
+
+    ConfigOptionSrc(
+        'encrypt_comments', bool, default=True,
+        extra_docs="""
+            Si activé (`True`), les contenus en commentaires pour chaque question (révélés lors de
+            la validation) sont cryptés lors de la création du qcm.
+        """,
+        yaml_desc = "Allow to disambiguate MCQ and SCQ when needed.",
     ),
 
     #-----------------------------------------------------------------------------
@@ -811,12 +750,12 @@ QCMS_CONFIG = SubConfigSrc(
 
 TERMS_CONFIG = SubConfigSrc(
     'terms',
-    docs = "Réglages spécifiques aux terminaux.",
+    extra_docs = "Réglages spécifiques aux terminaux.",
     elements = (
 
     ConfigOptionSrc(
         'cut_feedback', bool, default=True,
-        docs="""
+        extra_docs="""
             Si activé (`True`), les entrées affichées dans les terminaux sont tronquées si elles
             sont trop longues, afin d'éviter des problèmes de performances d'affichage des outils
             `jQuery.terminal`.
@@ -829,7 +768,7 @@ TERMS_CONFIG = SubConfigSrc(
 
     ConfigOptionSrc(
         'stdout_cut_off', int, default=200,
-        docs="""
+        extra_docs="""
             Nombre maximal de lignes restant affichées dans un terminal : si de nouvelles
             lignes sont ajoutées, les plus anciennes sont éliminées au fur et à mesure.
 
@@ -846,17 +785,6 @@ TERMS_CONFIG = SubConfigSrc(
                 tandis que l'option de troncature, `cut_feedback` évitera ce problème aussi.
         """,
         yaml_desc = "Maximum number of lines kept in terminals.",
-        # yaml_desc="Nombre de lignes maximales affichables dans les terminaux.",
-        # """
-        # Maximum number of lines displayed at once in a terminal. If more lines are printed, the
-        # lines at the top are removed.
-
-        # NOTE: jQuery.terminals become AWFULLY SLOW when the number of characters they display
-        # become somewhat massive. This option allows to limit these performances troubles, when
-        # the stdout is not truncated (see terminals upper right corner button). Also note that
-        # this option _does not_ limit the number of characters per line, so a frozen page can
-        # still occur, while the truncation feature will take care of that.
-        # """
     ),
 
     #--------------------------------------------------------------------------------
@@ -880,14 +808,14 @@ TERMS_CONFIG = SubConfigSrc(
 TESTING_CONFIG = SubConfigSrc(
     'testing',
     long_accessor = True,
-    docs = """
+    extra_docs = """
         Permet de paramétrer la page pour tester automatiquement tous les IDEs de la documentation.
     """,
     elements = (
 
     ConfigOptionSrc(
         'page', str, default="test_ides",
-        docs="""
+        extra_docs="""
             Nom de fichier markdown (avec ou sans l'extension `.md`) utilisé pour générer une page
             contenant le nécessaire pour tester de manière semi-automatisée tous les IDEs de
             la documentation.
@@ -900,17 +828,17 @@ TESTING_CONFIG = SubConfigSrc(
         yaml_desc = "Name of the IDEs testing page. Generated only if given."
     ),
     ConfigOptionSrc(
-        'include', str, conf_type=C.Choice(
-            (PmtTests.null, PmtTests.serve, PmtTests.site), default=PmtTests.null
-        ),
-        docs=f"""
+        'include', str,
+        conf_type = C.Choice(PageInclusion.VALUES[:3], default=PageInclusion.serve),
+        extra_docs = f"""
             Définit si la page de tests des IDEs doit être générée et de quelle façon.
             {'{{'}ul_li([
-                "`#!py '{PmtTests.null}'` : la page de tests n'est pas générée.",
-                "`#!py '{PmtTests.serve}'` : la page de tests est générée pendant `mkdocs
-                serve`, et est ajoutée automatiquement à la navigation.",
-                "`#!py '{PmtTests.site}'` : la page de tests est ajoutée sur le site construit,
-                mais n'apparaît pas dans la navigation.",
+                "`#!py '{PageInclusion.none}'` (_défaut_) : la page de tests n'est pas générée.",
+                "`#!py '{PageInclusion.serve}'` : la page de tests est générée pendant `mkdocs serve`,
+                et est ajoutée automatiquement à la navigation.",
+                "`#!py '{PageInclusion.site}'` : La page de tests est ajoutée au site construit,
+                mais n'y apparaît pas dans la navigation. Elle est aussi présente en `serve`
+                (page et navigation).",
             ]){'}}'}
         """,
         yaml_desc="""
@@ -919,11 +847,12 @@ TESTING_CONFIG = SubConfigSrc(
     ),
     ConfigOptionSrc(
         'load_buttons', bool, is_optional=True,
-        docs="""
-            Définit si le bouton pour charger l'ensemble des codes associés à un IDE de la page
-            des tests sera présent ou non.
-            <br>Le comportement par défaut dépend de la valeur de l'option {{ config_link(
-            'testing.include') }} :
+        extra_docs="""
+            Définit si le bouton pour charger l'ensemble des codes associés à un IDE de la page des tests
+            sera présent ou non.
+
+            Le comportement par défaut, avec {{ config_validator("testing.load_buttons", 1, val=None, yaml=1) }}
+            dépend de la valeur de l'option {{ config_link('testing.include') }} :
 
             * Pour {{ config_validator("testing.include", val="serve") }}, le bouton est présent par défaut.
             * Pour {{ config_validator("testing.include", val="site") }}, le bouton est absent par défaut.
@@ -932,8 +861,8 @@ TESTING_CONFIG = SubConfigSrc(
     ),
     ConfigOptionSrc(
         'empty_section_fallback', str,
-        conf_type = C.Choice(CASES_OPTIONS, default='skip'),
-        docs="""
+        conf_type = C.Choice(NamedTestCase.VALUES, default='skip'),
+        extra_docs="""
             Lorsque la page des tests des IDEs est construite et que la section à tester pour
             un IDE donné ne contient pas de code et que `{{config_validator("testing.empty_section_fallback", 1)}}`
             est définie, c'est cette "stratégie" qui sera utilisée à la place.
@@ -950,12 +879,67 @@ TESTING_CONFIG = SubConfigSrc(
 
 
 
+
+
+
+
+PLAYGROUND_CONFIG = SubConfigSrc(
+    'playground',
+    long_accessor = True,
+    extra_docs = """
+        Permet de paramétrer l'inclusion la page de développement/essais "playground".
+    """,
+    elements = (
+
+    ConfigOptionSrc(
+        'page', str, default="playground",
+        extra_docs="""
+            Nom de fichier markdown (avec ou sans l'extension `.md`) utilisé pour générer
+            la page de développement "playground" permettant de modifier en live toutes
+            les sections d'un IDE.
+
+            * La page n'est créée que si l'option `{{config_validator("playground.include")}}`
+            n'est pas à `#!yaml null`.
+            * Une erreur est levée si un fichier du même nom existe déjà.
+            * Une erreur est levée si le fichier n'est pas à la racine de la documentation.
+        """,
+        yaml_desc = "Name of the IDEs playground page. Generated only if given."
+    ),
+    ConfigOptionSrc(
+        'include', str,
+        conf_type = C.Choice(PageInclusion.VALUES, default=PageInclusion.serve),
+        extra_docs = f"""
+            Définit si la page de "playground" doit être générée et de quelle façon.
+            {'{{'}ul_li([
+                "`#!py '{PageInclusion.none}'` : la page n'est pas générée.",
+                "`#!py '{PageInclusion.serve}'` : la page est générée pendant `mkdocs serve`,
+                et est ajoutée automatiquement à la navigation.",
+                "`#!py '{PageInclusion.site}'` : La page de tests est ajoutée au site construit,
+                mais n'y apparaît pas dans la navigation. Elle est aussi présente en `serve`
+                (page et navigation).",
+                "`#!py '{PageInclusion.site_with_nav}'` : La page de tests est ajoutée au site
+                construit et en `serve`, et est visible dans la navigation dans les deux cas.",
+            ]){'}}'}
+        """,
+        yaml_desc="""
+            Define when and how to generate the playground page of the documentation.
+        """
+    ),
+    ),
+)
+
+
+
+
+
+
+
 # Kept as archive, but not used anymore:
 
 # OTHERS_CONFIG = SubConfigSrc(
 #     '_others',
-#     docs = "Réglages provenant de pyodide-mkdocs, mais qui ne sont plus utilisés.",
-#     dep_status = DeprecationStatus.unsupported,
+#     extra_docs = "Réglages provenant de pyodide-mkdocs, mais qui ne sont plus utilisés.",
+#     deprecation_status = DeprecationStatus.unsupported,
 #     elements = (
 #         ConfigOptionDeprecated('scripts_url', str),
 #         ConfigOptionDeprecated('site_root',   str),
