@@ -22,18 +22,19 @@ If not, see <https://www.gnu.org/licenses/>.
 
 
 import re
-from typing import  Dict, List, Optional
+from typing import  ClassVar, Dict, List, Optional, Set, Tuple, Union
 from functools import wraps
 from pathlib import Path
 
 from mkdocs.config.defaults import MkDocsConfig
-from mkdocs.exceptions import BuildError
 from mkdocs.structure.pages import Page
 from mkdocs.plugins import BasePlugin
 
+from pyodide_mkdocs_theme.pyodide_macros.tools_and_constants import DeprecationLevel, ScriptData
+
 
 from ...__version__ import __version__
-from ..exceptions import PyodideMacrosDeprecationError, PyodideMacrosError
+from ..exceptions import PmtInternalError, PmtMacrosDeprecationError, PmtMacrosError
 from ..messages import LangBase, Lang
 from ..messages.classes import JsDumper
 from ..pyodide_logger import logger
@@ -87,24 +88,34 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
     use_directory_urls: str = ConfigExtractor('_conf')
 
     # ARGS EXTRACTOR TOKEN
-    args_IDE_AUTO_RUN: bool = ConfigExtractor('config.args.IDE', prop='AUTO_RUN')
-    args_IDE_EXPORT:   bool = ConfigExtractor('config.args.IDE', prop='EXPORT')
-    args_IDE_LOGS:     bool = ConfigExtractor('config.args.IDE', prop='LOGS')
-    args_IDE_MAX:       int = ConfigExtractor('config.args.IDE', prop='MAX')
-    args_IDE_MAX_SIZE:  int = ConfigExtractor('config.args.IDE', prop='MAX_SIZE')
-    args_IDE_MERMAID:  bool = ConfigExtractor('config.args.IDE', prop='MERMAID')
-    args_IDE_MIN_SIZE:  int = ConfigExtractor('config.args.IDE', prop='MIN_SIZE')
-    args_IDE_MODE:      str = ConfigExtractor('config.args.IDE', prop='MODE')
-    args_IDE_REC_LIMIT: int = ConfigExtractor('config.args.IDE', prop='REC_LIMIT')
-    args_IDE_RUN_GROUP: str = ConfigExtractor('config.args.IDE', prop='RUN_GROUP')
-    args_IDE_SANS:      str = ConfigExtractor('config.args.IDE', prop='SANS')
-    args_IDE_SHOW:      str = ConfigExtractor('config.args.IDE', prop='SHOW')
-    args_IDE_STD_KEY:   str = ConfigExtractor('config.args.IDE', prop='STD_KEY')
-    args_IDE_TERM_H:    int = ConfigExtractor('config.args.IDE', prop='TERM_H')
-    args_IDE_TEST:      str = ConfigExtractor('config.args.IDE', prop='TEST')
-    args_IDE_TWO_COLS: bool = ConfigExtractor('config.args.IDE', prop='TWO_COLS')
-    args_IDE_WHITE:     str = ConfigExtractor('config.args.IDE', prop='WHITE')
-    args_IDE_py_name:   str = ConfigExtractor('config.args.IDE', prop='py_name')
+    args_IDE_AUTO_RUN:      bool = ConfigExtractor('config.args.IDE', prop='AUTO_RUN')
+    args_IDE_EXPORT:        bool = ConfigExtractor('config.args.IDE', prop='EXPORT')
+    args_IDE_ID:             int = ConfigExtractor('config.args.IDE', prop='ID')
+    args_IDE_LOGS:          bool = ConfigExtractor('config.args.IDE', prop='LOGS')
+    args_IDE_MAX:            int = ConfigExtractor('config.args.IDE', prop='MAX')
+    args_IDE_MAX_SIZE:       int = ConfigExtractor('config.args.IDE', prop='MAX_SIZE')
+    args_IDE_MERMAID:       bool = ConfigExtractor('config.args.IDE', prop='MERMAID')
+    args_IDE_MIN_SIZE:       int = ConfigExtractor('config.args.IDE', prop='MIN_SIZE')
+    args_IDE_MODE:           str = ConfigExtractor('config.args.IDE', prop='MODE')
+    args_IDE_REC_LIMIT:      int = ConfigExtractor('config.args.IDE', prop='REC_LIMIT')
+    args_IDE_RUN_GROUP:      str = ConfigExtractor('config.args.IDE', prop='RUN_GROUP')
+    args_IDE_SANS:           str = ConfigExtractor('config.args.IDE', prop='SANS')
+    args_IDE_SHOW:           str = ConfigExtractor('config.args.IDE', prop='SHOW')
+    args_IDE_STD_KEY:        str = ConfigExtractor('config.args.IDE', prop='STD_KEY')
+    args_IDE_TERM_H:         int = ConfigExtractor('config.args.IDE', prop='TERM_H')
+    args_IDE_TEST:           str = ConfigExtractor('config.args.IDE', prop='TEST')
+    args_IDE_TWO_COLS:      bool = ConfigExtractor('config.args.IDE', prop='TWO_COLS')
+    args_IDE_WHITE:          str = ConfigExtractor('config.args.IDE', prop='WHITE')
+    args_IDE_py_name: Tuple[str] = ConfigExtractor('config.args.IDE', prop='py_name')
+
+    args_composed_py_attrs:          str = ConfigExtractor('config.args.composed_py', prop='attrs')
+    args_composed_py_auto_title:    bool = ConfigExtractor('config.args.composed_py', prop='auto_title')
+    args_composed_py_name_only:     bool = ConfigExtractor('config.args.composed_py', prop='name_only')
+    args_composed_py_no_block:      bool = ConfigExtractor('config.args.composed_py', prop='no_block')
+    args_composed_py_py_name: Tuple[str] = ConfigExtractor('config.args.composed_py', prop='py_name')
+    args_composed_py_sections:       str = ConfigExtractor('config.args.composed_py', prop='sections')
+    args_composed_py_title:          str = ConfigExtractor('config.args.composed_py', prop='title')
+    args_composed_py_with_headers:  bool = ConfigExtractor('config.args.composed_py', prop='with_headers')
 
     args_figure_SHOW:       str = ConfigExtractor('config.args.figure', prop='SHOW')
     args_figure_admo_class: str = ConfigExtractor('config.args.figure', prop='admo_class')
@@ -128,53 +139,75 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
     args_multi_qcm_shuffle_questions: bool = ConfigExtractor('config.args.multi_qcm', prop='shuffle_questions')
     args_multi_qcm_tag_list_of_qs:     str = ConfigExtractor('config.args.multi_qcm', prop='tag_list_of_qs')
 
+    args_py_attrs:       str = ConfigExtractor('config.args.py', prop='attrs')
     args_py_auto_title: bool = ConfigExtractor('config.args.py', prop='auto_title')
+    args_py_name_only:  bool = ConfigExtractor('config.args.py', prop='name_only')
+    args_py_no_block:   bool = ConfigExtractor('config.args.py', prop='no_block')
     args_py_py_name:     str = ConfigExtractor('config.args.py', prop='py_name')
+    args_py_title:       str = ConfigExtractor('config.args.py', prop='title')
 
-    args_py_btn_AUTO_RUN:   bool = ConfigExtractor('config.args.py_btn', prop='AUTO_RUN')
-    args_py_btn_HEIGHT:      int = ConfigExtractor('config.args.py_btn', prop='HEIGHT')
-    args_py_btn_ICON:        str = ConfigExtractor('config.args.py_btn', prop='ICON')
-    args_py_btn_MERMAID:    bool = ConfigExtractor('config.args.py_btn', prop='MERMAID')
-    args_py_btn_REC_LIMIT:   int = ConfigExtractor('config.args.py_btn', prop='REC_LIMIT')
-    args_py_btn_RUN_GROUP:   str = ConfigExtractor('config.args.py_btn', prop='RUN_GROUP')
-    args_py_btn_SANS:        str = ConfigExtractor('config.args.py_btn', prop='SANS')
-    args_py_btn_SHOW:        str = ConfigExtractor('config.args.py_btn', prop='SHOW')
-    args_py_btn_SIZE:        int = ConfigExtractor('config.args.py_btn', prop='SIZE')
-    args_py_btn_TIP:         str = ConfigExtractor('config.args.py_btn', prop='TIP')
-    args_py_btn_TIP_SHIFT:   int = ConfigExtractor('config.args.py_btn', prop='TIP_SHIFT')
-    args_py_btn_TIP_WIDTH: float = ConfigExtractor('config.args.py_btn', prop='TIP_WIDTH')
-    args_py_btn_WHITE:       str = ConfigExtractor('config.args.py_btn', prop='WHITE')
-    args_py_btn_WIDTH:       int = ConfigExtractor('config.args.py_btn', prop='WIDTH')
-    args_py_btn_WRAPPER:     str = ConfigExtractor('config.args.py_btn', prop='WRAPPER')
-    args_py_btn_py_name:     str = ConfigExtractor('config.args.py_btn', prop='py_name')
+    args_py_btn_AUTO_RUN:               bool = ConfigExtractor('config.args.py_btn', prop='AUTO_RUN')
+    args_py_btn_HEIGHT:                  int = ConfigExtractor('config.args.py_btn', prop='HEIGHT')
+    args_py_btn_ICON:                    str = ConfigExtractor('config.args.py_btn', prop='ICON')
+    args_py_btn_ID:                      int = ConfigExtractor('config.args.py_btn', prop='ID')
+    args_py_btn_MERMAID:                bool = ConfigExtractor('config.args.py_btn', prop='MERMAID')
+    args_py_btn_REC_LIMIT:               int = ConfigExtractor('config.args.py_btn', prop='REC_LIMIT')
+    args_py_btn_RUN_GROUP:               str = ConfigExtractor('config.args.py_btn', prop='RUN_GROUP')
+    args_py_btn_SANS:                    str = ConfigExtractor('config.args.py_btn', prop='SANS')
+    args_py_btn_SHOW:                    str = ConfigExtractor('config.args.py_btn', prop='SHOW')
+    args_py_btn_SIZE:                    int = ConfigExtractor('config.args.py_btn', prop='SIZE')
+    args_py_btn_TIP:                     str = ConfigExtractor('config.args.py_btn', prop='TIP')
+    args_py_btn_TIP_SHIFT:               int = ConfigExtractor('config.args.py_btn', prop='TIP_SHIFT')
+    args_py_btn_TIP_WIDTH: Union[float, int] = ConfigExtractor('config.args.py_btn', prop='TIP_WIDTH')
+    args_py_btn_WHITE:                   str = ConfigExtractor('config.args.py_btn', prop='WHITE')
+    args_py_btn_WIDTH:                   int = ConfigExtractor('config.args.py_btn', prop='WIDTH')
+    args_py_btn_WRAPPER:                 str = ConfigExtractor('config.args.py_btn', prop='WRAPPER')
+    args_py_btn_py_name:          Tuple[str] = ConfigExtractor('config.args.py_btn', prop='py_name')
 
-    args_run_AUTO_RUN: bool = ConfigExtractor('config.args.run', prop='AUTO_RUN')
-    args_run_MERMAID:  bool = ConfigExtractor('config.args.run', prop='MERMAID')
-    args_run_REC_LIMIT: int = ConfigExtractor('config.args.run', prop='REC_LIMIT')
-    args_run_RUN_GROUP: str = ConfigExtractor('config.args.run', prop='RUN_GROUP')
-    args_run_SANS:      str = ConfigExtractor('config.args.run', prop='SANS')
-    args_run_SHOW:      str = ConfigExtractor('config.args.run', prop='SHOW')
-    args_run_WHITE:     str = ConfigExtractor('config.args.run', prop='WHITE')
-    args_run_py_name:   str = ConfigExtractor('config.args.run', prop='py_name')
+    args_run_AUTO_RUN:      bool = ConfigExtractor('config.args.run', prop='AUTO_RUN')
+    args_run_ID:             int = ConfigExtractor('config.args.run', prop='ID')
+    args_run_MERMAID:       bool = ConfigExtractor('config.args.run', prop='MERMAID')
+    args_run_REC_LIMIT:      int = ConfigExtractor('config.args.run', prop='REC_LIMIT')
+    args_run_RUN_GROUP:      str = ConfigExtractor('config.args.run', prop='RUN_GROUP')
+    args_run_SANS:           str = ConfigExtractor('config.args.run', prop='SANS')
+    args_run_SHOW:           str = ConfigExtractor('config.args.run', prop='SHOW')
+    args_run_WHITE:          str = ConfigExtractor('config.args.run', prop='WHITE')
+    args_run_py_name: Tuple[str] = ConfigExtractor('config.args.run', prop='py_name')
 
-    args_section_py_name: str = ConfigExtractor('config.args.section', prop='py_name')
-    args_section_section: str = ConfigExtractor('config.args.section', prop='section')
+    args_section_attrs:       str = ConfigExtractor('config.args.section', prop='attrs')
+    args_section_auto_title: bool = ConfigExtractor('config.args.section', prop='auto_title')
+    args_section_name_only:  bool = ConfigExtractor('config.args.section', prop='name_only')
+    args_section_no_block:   bool = ConfigExtractor('config.args.section', prop='no_block')
+    args_section_py_name:     str = ConfigExtractor('config.args.section', prop='py_name')
+    args_section_section:     str = ConfigExtractor('config.args.section', prop='section')
+    args_section_title:       str = ConfigExtractor('config.args.section', prop='title')
 
-    args_terminal_AUTO_RUN: bool = ConfigExtractor('config.args.terminal', prop='AUTO_RUN')
-    args_terminal_FILL:      str = ConfigExtractor('config.args.terminal', prop='FILL')
-    args_terminal_MERMAID:  bool = ConfigExtractor('config.args.terminal', prop='MERMAID')
-    args_terminal_REC_LIMIT: int = ConfigExtractor('config.args.terminal', prop='REC_LIMIT')
-    args_terminal_RUN_GROUP: str = ConfigExtractor('config.args.terminal', prop='RUN_GROUP')
-    args_terminal_SANS:      str = ConfigExtractor('config.args.terminal', prop='SANS')
-    args_terminal_SHOW:      str = ConfigExtractor('config.args.terminal', prop='SHOW')
-    args_terminal_TERM_H:    int = ConfigExtractor('config.args.terminal', prop='TERM_H')
-    args_terminal_WHITE:     str = ConfigExtractor('config.args.terminal', prop='WHITE')
-    args_terminal_py_name:   str = ConfigExtractor('config.args.terminal', prop='py_name')
+    args_sqlide_autoexec: bool = ConfigExtractor('config.args.sqlide', prop='autoexec')
+    args_sqlide_base:      str = ConfigExtractor('config.args.sqlide', prop='base')
+    args_sqlide_espace:    str = ConfigExtractor('config.args.sqlide', prop='espace')
+    args_sqlide_hide:     bool = ConfigExtractor('config.args.sqlide', prop='hide')
+    args_sqlide_init:      str = ConfigExtractor('config.args.sqlide', prop='init')
+    args_sqlide_sql:       str = ConfigExtractor('config.args.sqlide', prop='sql')
+    args_sqlide_titre:     str = ConfigExtractor('config.args.sqlide', prop='titre')
+
+    args_terminal_AUTO_RUN:      bool = ConfigExtractor('config.args.terminal', prop='AUTO_RUN')
+    args_terminal_FILL:           str = ConfigExtractor('config.args.terminal', prop='FILL')
+    args_terminal_ID:             int = ConfigExtractor('config.args.terminal', prop='ID')
+    args_terminal_MERMAID:       bool = ConfigExtractor('config.args.terminal', prop='MERMAID')
+    args_terminal_REC_LIMIT:      int = ConfigExtractor('config.args.terminal', prop='REC_LIMIT')
+    args_terminal_RUN_GROUP:      str = ConfigExtractor('config.args.terminal', prop='RUN_GROUP')
+    args_terminal_SANS:           str = ConfigExtractor('config.args.terminal', prop='SANS')
+    args_terminal_SHOW:           str = ConfigExtractor('config.args.terminal', prop='SHOW')
+    args_terminal_TERM_H:         int = ConfigExtractor('config.args.terminal', prop='TERM_H')
+    args_terminal_WHITE:          str = ConfigExtractor('config.args.terminal', prop='WHITE')
+    args_terminal_py_name: Tuple[str] = ConfigExtractor('config.args.terminal', prop='py_name')
 
     _pmt_meta_filename:                 str = ConfigExtractor('config.build')
     deprecation_level:                  str = ConfigExtractor('config.build')
     encrypted_js_data:                 bool = ConfigExtractor('config.build')
+    extra_pyodide_sections:       List[str] = ConfigExtractor('config.build')
     forbid_macros_override:            bool = ConfigExtractor('config.build')
+    ides_id_hash_mode:                  str = ConfigExtractor('config.build')
     ignore_macros_plugin_diffs:        bool = ConfigExtractor('config.build')
     limit_pypi_install_to:        List[str] = ConfigExtractor('config.build')
     load_yaml_encoding:                 str = ConfigExtractor('config.build')
@@ -184,6 +217,8 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
     python_libs:                  List[str] = ConfigExtractor('config.build')
     skip_py_md_paths_names_validation: bool = ConfigExtractor('config.build')
     tab_to_spaces:                      int = ConfigExtractor('config.build')
+    _activate_cache:                   bool = ConfigExtractor('config.build', deprecated=True)
+    _show_cache_refresh:               bool = ConfigExtractor('config.build', deprecated=True)
 
     ace_style_dark:                               str = ConfigExtractor('config.ides')
     ace_style_light:                              str = ConfigExtractor('config.ides')
@@ -198,8 +233,13 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
     forbid_corr_and_REMs_with_infinite_attempts: bool = ConfigExtractor('config.ides')
     forbid_hidden_corr_and_REMs_without_secrets: bool = ConfigExtractor('config.ides')
     forbid_secrets_without_corr_or_REMs:         bool = ConfigExtractor('config.ides')
+    remove_assertions_stacktrace:                bool = ConfigExtractor('config.ides')
     show_only_assertion_errors_for_secrets:      bool = ConfigExtractor('config.ides')
 
+    playground_include: str = ConfigExtractor('config.playground', prop='include')
+    playground_page:    str = ConfigExtractor('config.playground', prop='page')
+
+    encrypt_comments:                     bool = ConfigExtractor('config.qcms')
     forbid_no_correct_answers_with_multi: bool = ConfigExtractor('config.qcms')
 
     sequential_only: List[str] = ConfigExtractor('config.sequential', prop='only')
@@ -254,7 +294,33 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
     Built at runtime (depends on `macro_with_indents`)
     """
 
-    __case_url_cache: Dict[str,str]
+    __lvl_up_url_cache: Dict[str,str]
+
+    allowed_pmt_sections: Set[Union[ScriptData,str]] = ScriptData._MEMBERS
+    """
+    Automatically updated at runtime (on config swaps): contains all the PMT section names allowed
+    for the current page, considering the local value for `build.extra_pyodide_sections`.
+    """
+    allowed_pmt_sections_in_order: List[Union[ScriptData,str]] = ScriptData.VALUES
+    """
+    Automatically updated at runtime (on config swaps): contains all the PMT section names allowed
+    for the current page, considering the local value for `build.extra_pyodide_sections`.
+    """
+
+    pmt_sections_pattern: re.Pattern = ScriptData.build_pattern_from(ScriptData.gen_values(keep=True))
+    """
+    Automatically updated at runtime (on config swaps): pattern to identify `PYODIDE:{section}`
+    allowed in the current files (WITH ignore sections).
+    """
+
+    ACTIVATE_CACHE: ClassVar[bool] = False
+    """
+    ALWAYS `False`!
+
+    Kinda "placeholder" property, to keep the code using previously the cache logistic "as is",
+    but the related option (`build.activate_cache`) has been deprecated and will probably be
+    suppressed from the configuration later in time...
+    """
 
     #----------------------------------------------------------------------------
     # Also transferred to JS CONFIG:
@@ -267,17 +333,23 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
     lang:Lang = None
     language:str = 'fr'
 
+    py_snippets_stem: str = ".snippets"
+
 
     @property
     def directory_urls_is_false(self):
         """ Return True if config.use_directory_urls is false. """
         return not self._conf.use_directory_urls
 
+    @property
+    def use_sqlite_console(self):
+        """ Return True if "sqlite-console" is found in the MkDocsConfig. """
+        return 'sqlite-console' in self._conf.plugins
 
 
     # Override MacroPlugin
     def on_config(self, config:MkDocsConfig):       # pylint: disable=missing-function-docstring
-        self.__case_url_cache = {}
+        self.__lvl_up_url_cache = {}
         dct       = LangBase.get_langs_dct()
         self.lang = dct[self.language]
         PLUGIN_CONFIG_SRC.update_lang_defaults_with_current_lang(self)
@@ -308,7 +380,7 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
         # Raise if different macros are registered with the same name (unless allowed).
         # Note: the macro plugin creates a fresh dict on each on_config hook.
         if self.forbid_macros_override and name in self.macros:
-            raise PyodideMacrosError(
+            raise PmtMacrosError(
                 f'A macro named "{name}" has already been registered, possibly by the theme '
                 f'itself.\nPlease remove or rename the { name } macro is in the module: '
                 f'{ func.__module__ }'
@@ -335,7 +407,7 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
         """
         page = page or getattr(self, 'page', None)
         if not page:
-            raise BuildError("No page defined yet")
+            raise PmtInternalError("No page defined yet")
         return f"{ self.docs_dir_cwd_rel }/{ page.file.src_uri }"
 
 
@@ -349,12 +421,12 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
         @url: relative to the docs_dir (ex: "exercices/ ..."). If None, use self.page.url instead.
         """
         url = self.page.url if url is None else url
-        if url not in self.__case_url_cache:
+        if url not in self.__lvl_up_url_cache:
             page_loc:Path = self.docs_dir_path / url
             segments      = page_loc.relative_to(self.docs_dir_path).parts
             out           = ['..'] * ( len(segments) - self.directory_urls_is_false )
-            self.__case_url_cache[url] = '/'.join(out) or '.'
-        return self.__case_url_cache[url]
+            self.__lvl_up_url_cache[url] = '/'.join(out) or '.'
+        return self.__lvl_up_url_cache[url]
 
 
     def setup_ctx(self, page:Optional[Page], base_url:Optional[str]=None):
@@ -370,8 +442,9 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
             # This is for backward compatibility:
             #   - page may be None when building the 404 page
             #   - base_url must be defined for the 404 page
-            #   - page may also be None is other cases (typically, for dev_ops operations!),
-            #     then self._page must NOT be updated, but base_url doesn't matter
+            #   - page may also be None in other cases (typically, for dev_ops operations when
+            #     mkdocs is not running!), then self._page must NOT be updated, but base_url
+            #     doesn't matter
         self.base_url = self.level_up_from_current_page(page.url) if base_url is None and page else '.'
 
 
@@ -404,10 +477,10 @@ class BaseMaestro(BasePlugin[PyodideMacrosConfig]):
             )
             msg += (
                 "\nIf you absolutely need to pass the build right now, you can change the plugin option "
-                "build.deprecation_level to 'warn'."
+                f"build.deprecation_level to {DeprecationLevel.warn!r}."
             )
 
-        if self.deprecation_level == 'error' and not self._dev_mode:
-            raise PyodideMacrosDeprecationError(msg)
+        if self.deprecation_level == DeprecationLevel.error:
+            raise PmtMacrosDeprecationError(msg)
 
         logger.error(msg)
