@@ -25,12 +25,11 @@ If not, see <https://www.gnu.org/licenses/>.
 import re
 from textwrap import dedent
 from typing import Dict, List, Set, Tuple, Optional, Union
-from collections import Counter, defaultdict
+from collections import defaultdict
 from pathlib import Path
 
 
 from jinja2 import Environment
-from mkdocs.exceptions import BuildError
 from mkdocs.config.defaults import MkDocsConfig
 from mkdocs.structure.pages import Page
 from mkdocs.structure.files import Files
@@ -38,22 +37,23 @@ from mkdocs.structure.nav import Navigation
 from mkdocs.plugins import event_priority
 
 
+from ..exceptions import PmtInternalError, PyodideMkdocsError
 from ..tools_and_constants import (
     HIDDEN_MERMAID_MD,
     HtmlClass,
     IdeConstants,
     PageUrl,
     Prefix,
-    SequentialRun
 )
 from ..parsing import eat, compress_LZW
 from ..paths_utils import PathOrStr
 from ..pyodide_logger import logger
 from ..html_dependencies import HtmlDependencies, Block, DepKind
+from ..macros.ide_files_data import IdeFilesExtractor
 from .tools.maestro_tools import AutoCounter
 from .tools.macros_data import MacroData, IdeToTest, MacroDataRun
 from .tools.pages_and_macros_py_configs import MacroPyConfig, PageConfiguration
-from .config import PLUGIN_CONFIG_SRC
+from .config import PLUGIN_CONFIG_SRC, GIT_LAB_PAGES
 from .maestro_meta import MaestroMeta
 from .maestro_indent import MaestroIndent
 
@@ -80,21 +80,6 @@ class MaestroMacroManager(MaestroIndent):
     compteur_qcms: int = AutoCounter()
     """ Number of qcm or qcs in the docs """
 
-
-    generic_count: int = AutoCounter()
-    """
-    Number of empty terminals (per site count. Related to the terminal() macro).
-    Since there are no data associated to these in the locale storage (at least,
-    no _important_ data), this counter is used for everything.
-    """
-
-    ide_count: int = AutoCounter()
-    """
-    Number of empty IDEs (per site count. Used for IDE without python files).
-    Dedicated counter, for legacy reasons. Might break users' storage sometimes, but
-    this will provide more consistency with the evolution of the redactor's documentation.
-    """
-
     _editors_ids: Set[str]
     """
     Store the ids of all the created IDEs, to enforce their uniqueness.
@@ -110,13 +95,13 @@ class MaestroMacroManager(MaestroIndent):
     def on_config(self, config:MkDocsConfig):
 
         self.compteur_qcms = 0
-        self.generic_count = 0
-        self.ide_count     = 0
         self._editors_ids  = set()
         self._editors_paths_to_ids = defaultdict(set)
 
         super().on_config(config)   # pylint: disable=no-member
 
+        IdeFilesExtractor.propagate_any_refresh_needs()
+
 
     def get_qcm_id(self):
         return f"{ Prefix.py_mk_qcm_id_ }{ self.compteur_qcms :0>5}"
@@ -144,7 +129,7 @@ class MaestroMacroManager(MaestroIndent):
 
 
     def get_hdr_and_public_contents_from(
-        self, opt_path_or_content: Union[ Optional[Path], str ],
+        self, opt_path_or_content: Union[ Optional[Path], str ], apply_strip:bool=True,
     ) -> Tuple[str,str,str]:
         """
         Extract the header code and the public content from the given file (assuming it's a
@@ -166,7 +151,7 @@ class MaestroMacroManager(MaestroIndent):
         #   - len == 3 : ['', hdr, content]
 
         if len(lst) not in (1,3):
-            raise BuildError(
+            raise PyodideMkdocsError(
                 f"Wrong number of HDR/ENV tokens (found { len(lst)-1 }) in:\n"
                 f"{opt_path_or_content!s}"
             )
@@ -176,55 +161,65 @@ class MaestroMacroManager(MaestroIndent):
 
         # Make sure the old version of the pattern separating `code` from public `tests` is
         # still accepted (...or something close... :p ):
-        pm_pattern = f"# ?tests(?: ?:)?|{ self.lang.tests.as_pattern.pattern }"
-        tests_cuts = re.split(pm_pattern, content, flags=re.I)
+        pm_pattern = f"^# ?tests(?: ?:)?|{ self.lang.tests.as_pattern.pattern }"
+        tests_cuts = re.split(pm_pattern, content, flags=re.I|re.M)
 
         if len(tests_cuts)>2:
             pattern = self.lang.tests.as_pattern.pattern
-            raise BuildError(
+            raise PyodideMkdocsError(
                 f'Found more than one time the token {pattern!r} (case insensitive) in:'
                 f"\n{ opt_path_or_content }"
             )
 
         tests_cuts.append('')       # ensure always at least 2 elements
-        user_code, public_tests, *_ = map(str.strip, tests_cuts)
+        user_code, public_tests, *_ = map(str.strip, tests_cuts) if apply_strip else tests_cuts
 
         return hdr, user_code, public_tests
 
 
 
     def get_sibling_of_current_page(
-        self, partial_path: PathOrStr, *, tail:str="", rel_to_docs:bool=False
-    ):
+        self, partial_path: PathOrStr, *, tail:str="",
+    ) -> Optional[Path] :
         """
-        @env: BaseMaestro
-        @docs_dir
         Extract the current page from the env, then build a path name toward a file using
         the @partial_path (string name of slash separated string, or relative Path), starting
         from the same parent directory.
+
         If @tail is given, it is added to the name of the last segment of of the built path.
-        If @rel_to_docs is False, the result will be an absolute path of the file at documentation
-        building time. If @rel_to_docs is True, the returned path will be rooted at the docs_dir,
-        meaning the path will also be the valid relative path of the file on the built site.
 
         If @partial_path is an empty string or a Path without name, return None
+
+        WARNING:
+            * This function returns an absolute path, BUT...
+            * ...the output path is not normalized, meaning...
+            * identical target files using "back and forth" moves will return different output...
+            * And this kind of path is controlling the need for ID arguments or not
+
+        CCL: DON'T EVER NORMALIZE THE PATH HERE!! (huge breaking change)
         """
         path: Path = self.docs_dir_path / self.page.file.src_uri
-        out_path = self._get_sibling(path, partial_path, tail)
-        if out_path and rel_to_docs:
-            return out_path.relative_to(self.docs_dir_path)
+        out_path = self._get_sibling(path, partial_path, tail=tail)
         return out_path
 
 
     @staticmethod
-    def _get_sibling(src:Path, rel:PathOrStr, tail:str="") -> Optional[Path] :
-        """ Build a sibling of the given path file, replacing the current Path.name
-            with the given @rel element (path or str).
-            If @tail is given, it is added to the name of the built path.
-            Return None if:
-                - @src has no explicit parent (using '.' on the cwd will cause troubles)
-                - @rel is an empty string or is a Path without name property (empty IDE)
+    def _get_sibling(src:Path, rel:PathOrStr, *, tail:str="") -> Optional[Path] :
         """
+        Build a sibling of the given path file, replacing the current `Path.name` with the given
+        @rel element (path or str), and using the different kinds of logic allowed by the theme
+        to find the files:
+        1. Relative to "{src.parent}/" (priority)
+        2. Relative to "{src.parent}/scripts/{src.stem}/"
+
+        `@tail`: suffix to add to the file stem (if given).
+
+        Return None if:
+        - `@src` has no explicit parent (using '.' on the cwd will cause troubles)
+        - `@rel` is an empty string or is a Path without name property (empty IDE)
+        """
+        # NOTE: the docstring might be  outdated... (-> @rel info!?)
+
         if not src.name or not rel or isinstance(rel,Path) and not rel.name:
             return None
 
@@ -237,7 +232,6 @@ class MaestroMacroManager(MaestroIndent):
                 path = path.with_name( path.stem + tail )
             if path.is_file():
                 return path
-
         return None
 
 
@@ -296,12 +290,18 @@ class MaestroMacros(MaestroMacroManager):
     Store Python files info that should be updated (from PM to PMT format).
     """
 
+    macros_counters: Dict[str,AutoCounter] = None
+    """
+    Per page-wise counter of runners that are not linked to any python file.
+        Dict[ MACRO_NAME -> AutoCounter ]
+    """
+
 
     @property
     def current_page_config(self) -> PageConfiguration :
         """ Note: usable only once on_page_markdown has been called """
         if self.page.url not in self._pages_configs:
-            raise BuildError(
+            raise PmtInternalError(
                 f"PageConfiguration object for { self.file_location() } has not been created yet."
             )
         return self._pages_configs[self.page.url]
@@ -333,6 +333,7 @@ class MaestroMacros(MaestroMacroManager):
         marked as needing some.
         """
         self._pages_configs[self.page.url] = PageConfiguration(self)
+        self.macros_counters = defaultdict(AutoCounter)
 
         markdown_out = super().on_page_markdown(
             markdown, page, config, site_navigation=site_navigation, **kwargs
@@ -350,12 +351,13 @@ class MaestroMacros(MaestroMacroManager):
         self.all_macros_data.append(macro_data)
 
 
+
     def apply_macro(self, name, func, *args, **kwargs):            # pylint: disable=unused-argument
         """
         Apply all default arguments from the current config, register the related MacroData objects
         and manage them when needed.
         """
-        self.current_macro_data = None # reset
+        self.current_macro_data: Optional[MacroData] = None  # reset
 
         args, kwargs = PLUGIN_CONFIG_SRC.assign_defaults_to_args_and_macro_data(
             name, args, kwargs, self
@@ -370,26 +372,35 @@ class MaestroMacros(MaestroMacroManager):
 
     def file_location(self, page:Optional[Page]=None, *, full_macro=False, intro=False, all_in=False):
         """
-        Path to the current file, relative to the cwd, including the current running macro if any.
+        String info about to the current file, relative to the cwd, including the current running
+        macro if any.
 
         @full_macro: if True and this is a PMT macro currently running, add the complete arguments
         values to the message.
+
         @intro: if True, prepend the message with the sentence
+
+        @all_in: set both flags to True.
         """
         out = super().file_location(page)
         if self.current_macro_data:
             if all_in:
                 full_macro = intro = True
-            if intro:
-                out = f"\nHappened in file { out } in macro: { self._running_macro_name }."
-            if full_macro:
+            config = intro + 2* full_macro
+            if not config:
+                out += f" (in macro: { self._running_macro_name })"
+            if config&1:
+                out  = f"\nHappened in file { out } in macro: \033[31m{ self._running_macro_name }\033[0m."
+            if config&2:
                 lines = "\n\t".join(self.current_macro_data.args.show_as_list())
                 out += f"\nMacro arguments:\n\t{ lines }"
-            else:
-                out += f" (in macro: { self._running_macro_name })"
         return out
 
 
+    def log(self):
+        """ Shorthand for `self.file_location(all_in=True)`. """
+        return self.file_location(all_in=True)
+
 
     def on_env(self, env:Environment, config:MkDocsConfig, files:Files):
         from ..macros.ide_run_macro import AutoRun      # pylint: disable=C0415
@@ -407,8 +418,7 @@ class MaestroMacros(MaestroMacroManager):
             logger.info(
                 'You are currently using the outdated "multi-python files" setup of the original '
                 'pyodide-mkdocs prototype. You should update these files to match PMT requirements:'
-               f'{ files }\n(For more information, see '
-                'https://frederic-zinelli.gitlab.io/pyodide-mkdocs-theme/redactors/IDE-quick-guide/ ).'
+               f'{ files }\n(For more information, see { GIT_LAB_PAGES }redactors/IDE-quick-guide/ ).'
             )
 
 
@@ -429,23 +439,30 @@ class MaestroMacros(MaestroMacroManager):
         # needs the current page object:
         logger.debug(f"Add scripts + encrypt solutions and remarks in {self.file_location()}")
 
-        page_config = self._pages_configs[page and page.url]
-        chunks      = []
+        chunks       = []
+        page_config  = self._pages_configs[page and page.url]
+        is_tester    = page_config.has_need(DepKind.ides_test)
+        need_encrypt = (
+            page_config.has_need(DepKind.ide) and self.encrypt_corrections_and_rems
+            or page_config.has_need(DepKind.qcm_encrypt)
+        )
 
-        if page_config.has_need(DepKind.ides_test):
+        if is_tester:
             from ..macros.ide_tester import IdeTester   # pylint: disable=import-outside-toplevel
 
             logger.info("Create the tests for all the IDEs in the test page.")
             tests_content = IdeTester.build_html_for_tester(self, self._pages_with_ides_to_test)
             chunks = [page.content, tests_content]
 
-        elif page_config.has_need(DepKind.ide):
-            self.chunk_and_encrypt_solutions_and_rems(page.content, chunks)
+        elif need_encrypt:
+            self.chunk_and_encrypt_contents(page.content, chunks)
 
         else:
             chunks.append(page.content)
 
-        # Insert hidden spans to trigger the `{{ run(...) }}` related codes on JS subscriptions:
+        # Insert hidden spans to trigger the `{{ run(...) }}` related codes on JS subscriptions
+        # (not added through the md to avoid the md converter surrounding the html with <p> tags,
+        # messing up the md -> html rendering):
         chunks.extend(
             f'<span id="{ run_macro.storage_id }" class="{ HtmlClass.py_mk_hidden }"></span>'
             for run_macro in self._pages_with_run_macro[page.url]
@@ -467,7 +484,7 @@ class MaestroMacros(MaestroMacroManager):
             expected = DEV_TESTS[page.url]
             actual   = self.editor_font_size, self.python_libs
             if actual != expected:
-                raise BuildError(dedent(f"""
+                raise PmtInternalError(dedent(f"""
                     Wrong implementation of the CONFIG dumps: on_page_context(...)
                         Page: { page.url }
                         Tested: ides.editor_font_size, self.python_libs
@@ -541,7 +558,7 @@ class MaestroMacros(MaestroMacroManager):
 
 
 
-    def chunk_and_encrypt_solutions_and_rems(self, html:str, chunks:List[str]):
+    def chunk_and_encrypt_contents(self, html:str, chunks:List[str]):
         """
         Assuming it's known that the @page holds corrections and/or remarks:
             - Search for the encryption tokens
@@ -549,9 +566,6 @@ class MaestroMacros(MaestroMacroManager):
             - Once done for the whole page, replace the page content with the updated version.
         Encryption tokens are removed on the way.
         """
-        if not self.encrypt_corrections_and_rems:
-            chunks.append(html)
-            return
 
         entering = 0
         while entering < len(html):
