@@ -18,20 +18,23 @@ If not, see <https://www.gnu.org/licenses/>.
 """
 
 
-
+from inspect import signature, Parameter
+from functools import wraps
 from pathlib import Path
 
 
+from jinja2 import Environment
 from mkdocs.config.defaults import MkDocsConfig
 from mkdocs_macros.plugin import MacrosPlugin
-
+from mkdocs.structure.nav import Navigation
+from mkdocs.structure.files import Files
 
 
 
 from ...__version__ import __version__
 from ..pyodide_logger import logger
 from .tools import test_cases
-from .tools.macros_data import MacroData
+from .tools.macros_data import MacroData, MacroDataSqlide
 from .maestro_base import BaseMaestro
 from .maestro_files import MaestroFiles
 from .maestro_meta import MaestroMeta
@@ -78,16 +81,20 @@ class PyodideMacrosPlugin(
     """
 
 
+    def on_startup(self, command:str, dirty:bool):
+        self.in_serve = command == 'serve' #and False      # Uncomment to simulate a build during serve
+
+
+
     # Override
+    @logger.hook
     def on_config(self, config:MkDocsConfig):
         # --------------------------------------------------------------
         # pylint: disable=attribute-defined-outside-init
         # Section to always apply first:
 
-        self._conf    = config # done in MacrosPlugin, but also here because needed here or there
-        self.in_serve = config.dev_addr.host in config.site_url
-        self.language = config.theme['language']
-
+        self._conf            = config # done in MacrosPlugin, but also here because needed here or there
+        self.language         = config.theme['language']
         self.docs_dir_path    = Path(config.docs_dir)
         self.docs_dir_cwd_rel = self.docs_dir_path.relative_to(Path.cwd())
         # --------------------------------------------------------------
@@ -98,6 +105,24 @@ class PyodideMacrosPlugin(
         logger.info("Plugin configuration ready.")
 
 
+    #--------------------------------------------------------------------------
+    # Define all hooks and call their super method, making sure the decoration is always at the top level
+
+    @logger.hook
+    def on_files(self, files: Files, /, *, config: MkDocsConfig):
+        return super().on_files(files, config=config)
+
+    @logger.hook
+    def on_nav(self, nav: Navigation, /, *, config: MkDocsConfig, files: Files):
+        return super().on_nav(nav, config=config, files=files)
+
+    @logger.hook
+    def on_env(self, env:Environment, config:MkDocsConfig, files:Files):
+        return super().on_env(env, config=config, files=files)
+
+    @logger.hook
+    def on_post_build(self, config: MkDocsConfig):
+        return super().on_post_build(config)
 
     #--------------------------------------------------------------------------
 
@@ -112,18 +137,31 @@ class PyodideMacrosPlugin(
             IDEs.IDE,
             IDEs.IDEv,
             IDEs.IDE_tester,
+            IDEs.IDE_playground,
             IDEs.terminal,
             IDEs.py_btn,
             IDEs.run,
-            IDEs.section,
             figure.figure,
+            py_script.section,
+            py_script.composed_py,
             py_script.py,
             qcm.multi_qcm,
         ]
+
+        # Register all theme's macros:
         for func in macros:
             self.macro(func(self))
+
         self.macro(test_cases.Case)     # Not a macro, but needed in the jinja environment
 
+        # If mkdocs-sqlite-console is registered as plugin for the project, automatically add
+        # the macro to the environment (requires sqlite-console > 1.0.7)
+        if self.use_sqlite_console:
+            sql = self._conf.plugins['sqlite-console']
+            if hasattr(sql, 'sqlide'):      # Do not register earlier versions of the plugin (no macro support)
+                sqlide = self._conf.plugins['sqlite-console'].sqlide
+                self.macro(sqlide)
+
         # def macro_with_warning(func):
         #     macro = func(self)
         #     logged = False          # log once only per macro...
@@ -137,7 +175,7 @@ class PyodideMacrosPlugin(
         #         return macro(*a,**kw)
         #     return wrapper
 
-        # old_macros = []                 # Kept incase becomes useful again one day...
+        # old_macros = []                 # Kept in case becomes useful again one day...
         # for func in old_macros:
         #     self.macro( macro_with_warning(func) )
 
