@@ -34,9 +34,10 @@ If not, see <https://www.gnu.org/licenses/>.
 # See: https://github.com/mkdocs/mkdocs/discussions/3241
 
 
+from functools import wraps
 import logging
 import platform
-from typing import Any, MutableMapping, Tuple
+from typing import Any, Callable, MutableMapping, Tuple
 
 
 
@@ -58,7 +59,8 @@ class Logger(logging.LoggerAdapter):
         """
         super().__init__(logger_, {})
         self.prefix = prefix
-        if platform.system() != 'Windows':
+        self.not_on_win = platform.system() != 'Windows'
+        if self.not_on_win:
             color = 36 if color is None else color
             self.prefix = f"\033[{ color }m{ prefix }\033[0m"
 
@@ -76,6 +78,24 @@ class Logger(logging.LoggerAdapter):
         return f"{self.prefix}: {msg}", kwargs
 
 
+    def hook(self, func:Callable):
+        """
+        Automatically add `logger.info` calls around the decorated function/method.
+        """
+        in_msg  = f"[{ func.__name__ }]"
+        out_msg = f"[{ func.__name__ } done]"
+        if self.not_on_win:
+            in_msg,out_msg = map("\033[3;33m{}\033[0m".format, (in_msg,out_msg))
+
+        @wraps(func)
+        def wrapper(*a, **kw):
+            self.info(in_msg)
+            out = func(*a, **kw)
+            self.info(out_msg)
+            return out
+        return wrapper
+
+
 
 def get_plugin_logger(name: str, color:int=None) -> Logger:
     """Return a logger for plugins.
