@@ -23,7 +23,9 @@ import re
 from argparse import Namespace
 from dataclasses import dataclass, fields
 from pathlib import Path
-from typing import ClassVar, Dict, Tuple, Type, TypeVar, TYPE_CHECKING
+from typing import ClassVar, Dict, Iterable, Set, Tuple, Type, TypeVar, TYPE_CHECKING, Union
+
+from .exceptions import PmtInternalError, PmtMacrosInvalidArgumentError
 
 
 if TYPE_CHECKING:
@@ -59,6 +61,9 @@ class AutoDescriptor:
 # --------------------------------------------------------------------
 
 
+RUN_GROUP_SKIP = 'SKIP'
+""" Specific value when skipping an IDE in groups """
+
 ZIP_EXTENSION: str = 'zip'
 
 MACROS_WITH_INDENTS = set()
@@ -231,6 +236,9 @@ class Prefix:
     tester_ = AutoDescriptor()
     """ Extra prefix for ids of the tester IDE """
 
+    playground_ = AutoDescriptor()
+    """ Extra prefix for ids of the playground IDE """
+
     solution_ = AutoDescriptor()
     """ Extra prefix for the div holding the correction + REM """
 
@@ -383,6 +391,12 @@ class HtmlClass:
     py_mk_item_qcm = AutoDescriptor()
     """ Class identifying the ul and li items for one question """
 
+    py_mk_comment_qcm = AutoDescriptor()
+    """ Class identifying "details" admonition containing comments for one question. """
+
+    py_mk_encrypted_qcm = AutoDescriptor()
+    """ Class identifying encrypted content in the "details" admonition containing comments for one question. """
+
     qcm_shuffle = AutoDescriptor()
     """ The questions and items must be shuffled if present """
 
@@ -419,6 +433,10 @@ class MyEnum:
     VALUES: ClassVar[Tuple[str]] = ()
     """ Automatically store all the values of the given "enum" as a tuple. """
 
+    _MEMBERS: ClassVar[Set[str]] = set()
+    """
+    List of all the values in the current Enum (see `is_member`. Ignored fields aren't seen).
+    """
 
     @classmethod
     def gen_values(cls, keep=False):
@@ -426,7 +444,9 @@ class MyEnum:
         Yields the property names associated to each item of the "enum", unless their property
         name is in the class level SKIPPED_FIELD tuple.
 
-        (Note : ClassVar attributes are not listed through `fields`)
+        Notes:
+            - ClassVar attributes are not listed through `fields`.
+            - Works properly with inheritance.
         """
         return (
             getattr(cls, field.name)
@@ -434,6 +454,50 @@ class MyEnum:
                if keep or field.name not in cls.SKIPPED_FIELDS
         )
 
+    @classmethod
+    def has_member(cls, value:str) -> bool :
+        """ Return True if the given value is a member of the "Enum" """
+        return value in cls._MEMBERS
+
+    @classmethod
+    def check_all_members_valid(cls, actual:Iterable['T']) -> bool :
+        """
+        Return True if the given iterable only contains existing members.
+
+        DO NOT send in generators or they will be consumed!
+        """
+        if cls._MEMBERS.issuperset(actual):
+            return
+
+        raise PmtMacrosInvalidArgumentError(
+            f"Unknown members for { cls.__name__ }:\n"
+            f"  Invalid members: { ', '.join( actual - cls._MEMBERS ) }\n"
+            f"\n  Valid members are: { ', '.join( sorted(cls._MEMBERS) ) }"
+        )
+
+
+    @classmethod
+    def check_all_member_defined(cls, iterable:Iterable['T']) -> bool :
+        """
+        Check that all members are defined in iterable then return it.
+        DO NOT send in generators!
+        """
+        actual = set(iterable)
+        if cls._MEMBERS != actual:
+            missing = cls._MEMBERS - actual
+            extra   = actual - cls._MEMBERS
+            msg = [
+                f"Invalid data structure déclaration based on { cls.__name__ }.\n",
+                f"  Missing members: { ', '.join(missing) }\n" * bool(missing),
+                f"  Invalid members: { ', '.join(extra) }\n" * bool(extra),
+                f"\n  Valid members are: { ', '.join(sorted(cls._MEMBERS)) }"
+            ]
+            raise PmtInternalError(''.join(msg))
+        return iterable
+
+
+
+
 
 
 T = TypeVar("T", bound=Type[MyEnum])
@@ -441,6 +505,7 @@ T = TypeVar("T", bound=Type[MyEnum])
 def dataclass_with_VALUES(cls:Type[T]) -> Type[T]:
     cls = dataclass(cls)
     cls.VALUES = tuple(cls.gen_values())
+    cls._MEMBERS = set(cls.VALUES)
     return cls
 
 
@@ -470,15 +535,59 @@ class ScriptSection(MyEnum):
     SKIPPED_FIELDS: ClassVar[Tuple[str]] = ('ignore',)
     ignore:    'ScriptSection' = AutoDescriptor()
 
+    # WARNING: keep the declaration order -> that's specification for `## {{ py_name:* }}` inclusions
     env:       'ScriptSection' = AutoDescriptor()
     env_term:  'ScriptSection' = AutoDescriptor()
-    user:      'ScriptSection' = AutoDescriptor("code")
+    code:      'ScriptSection' = AutoDescriptor()
     corr:      'ScriptSection' = AutoDescriptor()
     tests:     'ScriptSection' = AutoDescriptor()
     secrets:   'ScriptSection' = AutoDescriptor()
     post_term: 'ScriptSection' = AutoDescriptor()
     post:      'ScriptSection' = AutoDescriptor()
 
+    @classmethod
+    def gen_showing(cls):
+        yield from ( (section, f'PYODIDE:{ section }') for section in ScriptSection.VALUES)
+
+    @classmethod
+    def build_pattern_from(cls, sections:Iterable[Union['ScriptData',str]]) -> re.Pattern:
+        return re.compile(rf'PYODIDE[\t ]*:[\t ]*({ "|".join(sections) })\b')
+
+
+
+
+@dataclass_with_VALUES
+class ScriptData(ScriptSection):
+    """
+    Python sections & REM contents.
+    """
+
+    REM:     'ScriptData' = AutoDescriptor()
+    VIS_REM: 'ScriptData' = AutoDescriptor()
+
+    @classmethod
+    def gen_showing(cls):
+        yield from super().gen_showing()
+        yield from ( (section, f'{ section } content') for section in (cls.REM, cls.VIS_REM))
+
+
+@dataclass_with_VALUES
+class ScriptDataWithRemPaths(ScriptData):
+    """
+    Python sections, REM contents & REM paths.
+    """
+
+    REM_PATH:     'ScriptDataWithRemPaths' = AutoDescriptor()
+    VIS_REM_PATH: 'ScriptDataWithRemPaths' = AutoDescriptor()
+
+    @classmethod
+    def gen_showing(cls):
+        yield from super().gen_showing()
+        yield from ( (section, f'{ section } path') for section in (cls.REM_PATH, cls.VIS_REM_PATH))
+
+
+
+
 
 
 
@@ -536,9 +645,56 @@ class PmtPyMacrosName(MyEnum):
     py_btn:     'PmtPyMacrosName' = AutoDescriptor()
     run:        'PmtPyMacrosName' = AutoDescriptor()
 
-    SKIPPED_FIELDS: ClassVar[Tuple[str]] = ('IDE_tester',)
     IDE_tester: 'PmtPyMacrosName' = AutoDescriptor()
+    IDE_playground: 'PmtPyMacrosName' = AutoDescriptor()
+    SKIPPED_FIELDS: ClassVar[Tuple[str]] = 'IDE_tester', 'IDE_playground'
 
+    @classmethod
+    def is_ide(cls, name:str):
+        """
+        Tells if the macro involved matches the IDE_tester one or not.
+
+        NOTES:
+            - The method could get names from any macros, also containing IDE in their name
+              (like... IDE_py!) so the check must be strict.
+            - Not implemented on the IdeManager hierarchy, because this is also needed elsewhere
+              in the code base.
+        """
+        return name in (cls.IDE, cls.IDEv, cls.IDE_tester, cls.IDE_playground)
+
+    @classmethod
+    def get_macro_data_config_for(cls, name:str):
+        """
+        Transform any PMT macro name into its most generic version, and tells if the macro
+        involved matches the IDE_tester one or not.
+        """
+        name = cls.IDE if cls.is_ide(name) else name
+        in_macros_data = name not in (cls.IDE_tester, cls.IDE_playground)
+        return name, in_macros_data
+
+
+
+
+
+@dataclass_with_VALUES
+class HashPathMode(MyEnum):
+    """
+    Define how the IDEs html id hashes are computed from their python file location (when
+    they have one).
+    """
+
+    legacy: 'HashPathMode' = AutoDescriptor()
+    """ Use the absolute (unresolved) file path as base for the string to hash """
+
+    relative: 'HashPathMode' = AutoDescriptor()
+    """
+    Use the path relative to the docs_dir, to avoid any impact of a change of "location" of
+    the files on the machine building the site (local, pipeline, ...).
+    """
+
+    @classmethod
+    def is_legacy(cls, env:'PyodideMacrosPlugin'):
+        return env.ides_id_hash_mode == cls.legacy
 
 
 
@@ -570,6 +726,22 @@ class SequentialFilter(MyEnum):
 
 
 
+@dataclass_with_VALUES
+class NamedTestCase(MyEnum):
+    """
+    Identifiers for the default Case objects/configs.
+    """
+    none:     'NamedTestCase' = AutoDescriptor("")
+    skip:     'NamedTestCase' = AutoDescriptor()
+    fail:     'NamedTestCase' = AutoDescriptor()
+    code:     'NamedTestCase' = AutoDescriptor()
+    corr:     'NamedTestCase' = AutoDescriptor()
+    human:    'NamedTestCase' = AutoDescriptor()
+    no_clear: 'NamedTestCase' = AutoDescriptor()
+
+
+
+
 @dataclass_with_VALUES
 class SequentialRun(MyEnum):
     """
@@ -587,15 +759,6 @@ class SequentialRun(MyEnum):
 class MacroShowConfig(MyEnum):
     """ Possible values for the SHOW argument of the various macros. """
 
-    SKIPPED_FIELDS: ClassVar[Tuple[str]] = ('python', 'all')
-
-    python: 'MacroShowConfig' = AutoDescriptor()
-    """ Show python code sections (ONLY for ide_manager related macros) """
-
-    all: 'MacroShowConfig' = AutoDescriptor()
-    """ Show args and python code sections (ONLY for ide_manager related macros) """
-
-
     none: 'MacroShowConfig' = AutoDescriptor("")
     """ Default value: show nothing"""
 
@@ -605,6 +768,21 @@ class MacroShowConfig(MyEnum):
 
 
 
+@dataclass_with_VALUES
+class RunnersShowConfig(MacroShowConfig):
+
+    python: 'RunnersShowConfig' = AutoDescriptor()
+    """ Show python code sections (ONLY for runners related macros) """
+
+    contents: 'RunnersShowConfig' = AutoDescriptor()
+    """ Show python code sections and REMs content (ONLY for runners related macros) """
+
+    all: 'RunnersShowConfig' = AutoDescriptor()
+    """ Show args and python code sections (ONLY for runners related macros) """
+
+
+
+
 
 
 
@@ -624,17 +802,45 @@ class P5BtnLocation(MyEnum):
 
 
 
+@dataclass_with_VALUES
+class DecreaseAttemptsMode(MyEnum):
+    """
+    Possible HTML classes and arguments names for p5 animations buttons (in figure macros)
+    """
+    editor:  'DecreaseAttemptsMode' = AutoDescriptor()
+    public:  'DecreaseAttemptsMode' = AutoDescriptor()
+    secrets: 'DecreaseAttemptsMode' = AutoDescriptor()
+
+
+
+
+
+
+
 
 @dataclass_with_VALUES
-class PmtTests(MyEnum):
+class PageInclusion(MyEnum):
     """
     StrEnum-like object, referencing the possible options for the testing plugin's sub-config.
     """
-    null:  'PmtTests' = AutoDescriptor()
-    serve: 'PmtTests' = AutoDescriptor()
-    site:  'PmtTests' = AutoDescriptor()
+    none:          'PageInclusion' = AutoDescriptor()
+    serve:         'PageInclusion' = AutoDescriptor()
+    site:          'PageInclusion' = AutoDescriptor()
+    site_with_nav: 'PageInclusion' = AutoDescriptor()
 
 
+    @classmethod
+    def is_built(cls, value:'PageInclusion', env:'PyodideMacrosPlugin'):
+        return value != PageInclusion.none and (
+            env.in_serve or value != PageInclusion.serve
+        )
+
+    @classmethod
+    def is_in_nav(cls, value:'PageInclusion', env:'PyodideMacrosPlugin'):
+        return value != PageInclusion.none and (
+            env.in_serve or value == PageInclusion.site_with_nav
+        )
+
 
 
 
@@ -670,3 +876,12 @@ class DebugConfig(MyEnum):
 
     check_global_json_dump: bool = False
     """ If True, check that the JSON dump for PAGE_IDES_CONFIG is valid """
+
+
+
+
+
+@dataclass_with_VALUES
+class DeprecationLevel(MyEnum):
+    error: 'DeprecationLevel' = AutoDescriptor()
+    warn:  'DeprecationLevel' = AutoDescriptor()
