@@ -21,6 +21,16 @@ If not, see <https://www.gnu.org/licenses/>.
 import { jsLogger } from 'jsLogger'
 
 
+/**Isolated version, to make sure it doesn't clash with the original one from
+ * mathjax-libs.js.
+ * */
+export function perennialMathJaxUpdate(){
+    window.MathJax.startup.output.clearCache()
+    window.MathJax.typesetClear()
+    window.MathJax.texReset()
+    window.MathJax.typesetPromise()
+}
+
 
 
 
@@ -36,6 +46,30 @@ export function getTheme(){
 
 
 
+export function getIdeOptions(ide={}){
+    // https://github.com/ajaxorg/ace/wiki/Configuring-Ace
+    return {
+        autoScrollEditorIntoView: false,
+        copyWithEmptySelection:   true,               // active alt+flèches pour déplacer une ligne, aussi
+        enableBasicAutocompletion:true,
+        enableLiveAutocompletion: false,
+        enableSnippets:           true,
+        tabSize:                  4,
+        useSoftTabs:              true,               // Spaces instead of tabs
+        navigateWithinSoftTabs:   false,              // this is _fucking_ actually "Atomic Soft Tabs"...
+        printMargin:              false,              // hide ugly margins...
+        maxLines:                 ide.maxIdeLines ?? 30,
+        minLines:                 ide.minIdeLines ?? 10,
+        mode:                     "ace/mode/python",
+        theme:                    getTheme(),
+        fontSize:                 CONFIG.editorFontSize,
+        fontFamily:               [CONFIG.editorFontFamily, "Monaco", "Menlo", "Ubuntu Mono", "Consolas",
+                                   "Source Code pro", "source-code-pro", "monospace"],   // Fix apple troubles...
+    }
+}
+
+
+
 
 /**Randomly pick a value from an array.
  * @throws Error if the array is empty.
@@ -114,6 +148,10 @@ export const withPyodideAsyncLock = (_=>{
 
 
 
+
+
+
+
 /**Allow to delay the executions of various functions, until the pyodide environment
  * is ready.
  *
@@ -124,19 +162,20 @@ export const withPyodideAsyncLock = (_=>{
  *       that are actually locked (otherwise, deadlock!).
  * */
 export const waitForPyodideReady = async()=>{
-    LOGGER_CONFIG.ACTIVATE && jsLogger("[Wait4Pyodide] - ...")
 
     const maxWaitingTime = 20  // in seconds
     const attempts = 80
     const step_ms = Math.round(1000 * maxWaitingTime / attempts)
 
+    LOGGER_CONFIG.ACTIVATE && jsLogger(`[Wait4Pyodide] with ${ step_ms } ms in between attempts`)
     let counter = 0
     while(!CONFIG.pyodideIsReady){
         await sleep(step_ms);
-        if(++counter == attempts){
+        counter++
+        LOGGER_CONFIG.ACTIVATE && jsLogger("[Wait4Pyodide] -", counter,'/',attempts, "attempts")
+        if(counter == attempts){
             throw new Error(`Couldn't access to pyodide environment in time (${maxWaitingTime}s)`)
         }
-        LOGGER_CONFIG.ACTIVATE && jsLogger("[Wait4Pyodide] -", counter,'/',attempts, "attempts")
     }
 }
 
@@ -237,7 +276,6 @@ export function subscribeWhenReady(waitId, callback, options={}){
         return ()=>{ CONFIG.subscriptionReady[waitId]=true }
     }
 }
-// TOKEN: end subscribeWhenReady
 
 
 
@@ -314,6 +352,24 @@ export function isDark(){
 
 
 
+export function renderMermaidGraphs(removeCodeTags=true){
+    if(removeCodeTags){
+        $('pre.mermaid').each(function(){
+            const code = $(this).children('code')
+            if(code.length){
+                code.replaceWith(code.text())
+            }
+        })
+    }
+    setTimeout(async ()=>{
+        try{
+            await mermaid.run()
+        }catch(e){}
+    })
+}
+
+
+
 
 
 
@@ -769,6 +825,29 @@ export function buildJqHistoryBtn(
 
 
 
+// ATTEMPT. Not finished -> archived just in case...
+// export function fileContentToDom(content, mime, readerMethod, domQuery, domTemplate, templateRepl="{}", domAppend=false){
+//     const elt = $(domQuery)
+//     if(!domAppend) elt.html("")
+
+//     const reader = new FileReader()
+//     reader.abort = function(e){
+//       console.log('ERROR!')
+//     }
+//     reader.onload = function(e){
+//       const data = e.target.result
+//       const html = domTemplate.replace(templateRepl, data)
+//       elt.append(html)
+//     }
+
+//     const blob = new Blob([content], {type: mime})
+//     reader[readerMethod](blob)
+//   }
+
+
+
+
+
 
 /**Apply a "download" action, given the content, name and type of the file to download.
  * See https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Common_types
@@ -1008,7 +1087,7 @@ export class RunningProfile {
       name: profile,
       isTermCmd:  profile.includes(RunningProfile.PROFILE.cmd),
       isPlaying:  profile.includes(RunningProfile.PROFILE.play),
-      isChecking: profile.includes(RunningProfile.PROFILE.validate),
+      isValidating: profile.includes(RunningProfile.PROFILE.validate),
       isTesting:  profile.includes(RunningProfile.PROFILE.testing),
     })
   }
@@ -1037,4 +1116,5 @@ globalThis.config        = config
 globalThis.isDark        = isDark
 globalThis.downloader    = downloader
 globalThis.uploader      = uploader
-globalThis.uploaderAsync = uploaderAsync
\ No newline at end of file
+globalThis.uploaderAsync = uploaderAsync
+// globalThis.fileContentToDom = fileContentToDom
