@@ -18,12 +18,12 @@ If not, see <https://www.gnu.org/licenses/>.
 */
 
 
-// Must be defined to fulfill the legacy subscription logistic:
+
+// (dummy version, for backward compatibility)
 function mathJaxUpdate(){}
 
-subscribeWhenReady('MathJax')
-
 
+// You can change the configuration as you like, as long as the startup property stays unchanged
 window.MathJax = {
   startup: {
     ready: () => {
@@ -43,7 +43,7 @@ window.MathJax = {
     processEnvironments: true,
   },
   options: {
-    ignoreHtmlClass: ".*|",
-    processHtmlClass: "arithmatex",
+    ignoreHtmlClass: "(?!arithmatex|md-nav__item)",
+    processHtmlClass: "arithmatex|md-nav__item"
   },
 }
