@@ -29,7 +29,84 @@ export const pyodideFeatureCode=(()=>{
 
     const PYODIDE_SNIPPETS = {
 
-    autoRunCleaner: `__builtins__.auto_run.clean()`,
+
+    /**Creates the PyodideConsole, which is the runtime tool behind any "terminal", and setup
+     * the JS routines so that Runners can interact with it.
+     *
+     * https://pyodide.org/en/stable/usage/api/python-api/console.html#pyodide.console.PyodideConsole
+     *
+     *
+     * Routines are stored in `globalThis.pyFuncs`, which is also `PyodideSectionsRunner.pyFuncs`.
+     * The global version will be deleted after use, unless `CONFIG._devMode` is true.
+     * The available routines are:
+     *
+     *      - `redirect_cmd(cmd:string)` to run commands written by a user in the terminal.
+     *
+     *      - `complete(cmd:string)` to ask for an array of all the possible auto-completion
+     *         suggestions for the given command (currently typed in the terminal by a user).
+     *
+     *      - `clear_console()` to cancel the PyodideConsole current buffer, if ever needed
+     *        (for example: an incomplete command triggering an error about imports => the previous
+     *         parts of the command need to be cancelled).
+     *
+     * Archive note: in earlier implementations of pyodideStart, `__builtins__` was behaving here like
+     * a dict instead of an object because pyodide.runPython(...) was run using a dedicated namespace
+     *  object (second argument).
+     * */
+    pyodideStart: `
+def _hack_start_pyodide():
+    import js
+    import __main__
+    from pyodide.console import PyodideConsole, repr_shorten
+    from pyodide.ffi import to_js
+
+
+    # Globals initializations:
+    __builtins__._ = None
+    __builtins__.__USER_CMD__ = __builtins__.__USER_CODE__ = ""
+
+    # Store some original python functions:
+    for name in 'input print help'.split():
+        src_func = getattr(__builtins__, name)
+        setattr(__builtins__, f"__{ name }_src__", src_func)
+
+
+    py_console = PyodideConsole(__main__.__dict__)
+
+
+    async def redirect_cmd(cmd):
+
+        fut = py_console.push(cmd)
+        state = fut.syntax_check
+
+        if state != 'incomplete':
+            res = await fut             # Raising is desired if error
+            if res is not None:
+                __builtins__._ = res
+                if js.config().cutFeedback:
+                    res = repr_shorten(res)
+                print(res)
+
+        return state
+
+    def complete(cmd:str):
+        # Avoids some crashes on autocompletion requests on previous results of the autocompletion...
+        cmd = cmd.rstrip('(')
+        out = py_console.complete(cmd)
+        return to_js(out[0])
+
+    def clear_console():
+        py_console.buffer = []
+
+
+    # Proxies created once only per page, so don't care about destroying them:
+    js.pyFuncs.redirect_cmd = to_js(redirect_cmd)
+    js.pyFuncs.clear_console = to_js(clear_console)
+    js.pyFuncs.complete = to_js(complete)
+
+_hack_start_pyodide()
+del _hack_start_pyodide     # (no auto_run tool yet)
+`,
 
     autoRun: `
 def _hack_auto_run():
@@ -164,6 +241,9 @@ del _hack_auto_run
 `,
 
 
+    autoRunCleaner: `__builtins__.auto_run.clean()`,
+
+
     version: `
 def version(get_version=False):
     """ Print (also returns) the current version number of pyodide-mkdocs-theme. """
@@ -229,7 +309,7 @@ def _hack_io_stuff():
         print(re.sub(r"\x08.", "", doc))
 
 
-    PMT_TOOLS = ['p5', 'vis_network']
+    PMT_TOOLS = ['p5', 'vis', 'vis_network']      # GENERATED
 
 
     @wraps_builtin
@@ -260,8 +340,12 @@ def _hack_io_stuff():
         try:
             js.config().termMessage(key, msg, format or 'none', False, new_line)
         except Exception as e:
-            stripped_js_err = str(e)[ len('Error: '): ]
-            raise ValueError(stripped_js_err) from None
+            msg = str(e)
+            target = 'Error: '      # Error might be Error, or PythonError
+            i = msg.find(target)
+            if i<0: i = -len(target)
+            stripped_js_err = msg[ i+len(target): ]
+            raise Exception(stripped_js_err) from None
 `,
 
 
@@ -439,7 +523,7 @@ def _hack_uploader_downloader():
 `,
 
 
-    // GENERATED (modify the root project file instead)
+    // GENERATED (modify the root project file instead and/or ./python_devops/_js_codes_updates.py)
     pyodidePlot: `
 @auto_run
 def _hack_plot():
@@ -450,9 +534,6 @@ def _hack_plot():
     from typing import Callable, Sequence
 
 
-    import js
-    matplotlib.use("module://matplotlib_pyodide.html5_canvas_backend")
-
 
     class PyodidePlot:
         """
@@ -500,7 +581,7 @@ def _hack_plot():
             Close any previously created figure, then setup the current run to draw in
             the div tag targeted by the current instance.
             If keep_fig is set to True, the automatic \`Figure N\` will be kept, above the
-            drawn figure.
+            next drawn figure.
             """
             for _ in plt.get_fignums():
                 plt.close()
@@ -510,7 +591,7 @@ def _hack_plot():
             js.document.pyodideMplTarget = div
             div.textContent = ""
             if not keep_fig:
-                plt.gcf().canvas.manager.set_window_title('')
+                self.clear_current_figure_title()
             return plt
 
         refresh = target        # backward compatibility
@@ -572,6 +653,35 @@ def _hack_plot():
             return out
 
 
+        @classmethod
+        def clear_current_figure_title(cls):
+            """ Remove the automatic title added through matplotlib.pyplot. """
+            plt.gcf().canvas.manager.set_window_title('')
+
+
+        @classmethod
+        def interactive_backend(cls):
+            """ Use interactive html5 backend to draw figures """
+            matplotlib.use("module://matplotlib_pyodide.html5_canvas_backend")
+
+        @classmethod
+        def wasm_backend(cls):
+            """ Use wasm backend rendering to draw static figures """
+            matplotlib.use("module://matplotlib_pyodide.wasm_backend")
+
+        @classmethod
+        def matplotlib_backend(cls):
+            """ Use interactive html5 backend to draw figures """
+            cls.interactive_backend()
+
+        @classmethod
+        def sympy_backend(cls):
+            """ Use wasm backend rendering to draw static figures """
+            cls.wasm_backend()
+
+
+    import js
+    PyodidePlot.matplotlib_backend()
 
 # THIS IS A PLOT TOKEN
     __builtins__.PyodidePlot = PyodidePlot      # don't use the decorator because auto-updated code
@@ -594,6 +704,7 @@ def _hack_mermaid():
             if debug: print(code)
             code = f'<pre class="mermaid">{ code }</pre>'
             js.document.getElementById(div_id).innerHTML = code
+            js.config().calledMermaid = True
 
         return to_mermaid
 
