@@ -20,13 +20,14 @@ If not, see <https://www.gnu.org/licenses/>.
 
 import { jsLogger } from 'jsLogger'
 import {
+  RunningProfile,
   decompressPagesIfNeeded,
   noStorage,
+  renderMermaidGraphs,
   sleep,
   unEscapeSquaredBrackets,
   withPyodideAsyncLock,
   youAreInTroubles,
-  RunningProfile,
 } from 'functools'
 import { _DUMMY } from 'process_and_gui'   // Enforce dependencies order (if ever a runner is needed)
 
@@ -85,7 +86,6 @@ class PyodideSectionsRunnerBase {
   get hasCorrBtn()        { return this.data.has_corr_btn }
   get hasCounter()        { return this.data.has_counter }
   get hasRevealBtn()      { return this.data.has_reveal_btn }
-  get isEncrypted()       { return this.data.is_encrypted }
   get isVert()            { return this.data.is_vert }
   get maxIdeLines()       { return this.data.max_ide_lines }
   get minIdeLines()       { return this.data.min_ide_lines }
@@ -99,6 +99,7 @@ class PyodideSectionsRunnerBase {
   get pypiWhite()         { return this.data.pypi_white }
   get pythonLibs()        { return this.data.python_libs }
   get recLimit()          { return this.data.rec_limit }
+  get removeAssertionsStacktrace(){ return this.data.remove_assertions_stacktrace }
   get runGroup()          { return this.data.run_group }
   get secretTests()       { return this.data.secret_tests }
   get seqRun()            { return this.data.seq_run }
@@ -120,7 +121,7 @@ class PyodideSectionsRunnerBase {
 
 
     this.id = id
-    this.data = this._prepareData(PAGE_IDES_CONFIG[id])   // Inner mutable object holding infos coming from mkdocs
+    this.data = this._dataPostConversion(PAGE_IDES_CONFIG[id])   // Inner mutable object holding infos coming from mkdocs
     this.getCodeToTest = ()=>"" // If no editor, nothing to test...
     this.running = undefined    // RunningProfile.build output (object of boolean flags, mostly)
     this.allowPrint = true      // Control stdout display during executions
@@ -141,7 +142,9 @@ class PyodideSectionsRunnerBase {
   }
 
 
-  _prepareData(data){
+  /**Apply any desired data conversion on the info coming directly from mkdocs, before they are used.
+   * */
+  _dataPostConversion(data){
     data.python_libs = new Set(data.python_libs)
     return data
   }
@@ -182,7 +185,7 @@ class PyodideSectionsRunnerBase {
   /**Actions to perform when the element becomes "visible" in the page.
    * Here "visible" means "not hidden", CSS-wise (see `=== "tabbed"`, typically).
    * */
-  makeUpYourGui(){
+  makeUpYourGui(){        // Cap overload
     return this.isGuiCompliant = true
   }
 
@@ -267,6 +270,7 @@ class PyodideSectionsRunnerBase {
       if(eventOrCmd && eventOrCmd.preventDefault) eventOrCmd.preventDefault()
       LOGGER_CONFIG.ACTIVATE && jsLogger(loggerName)
 
+      CONFIG.calledMermaid = false
       this.makeDirty(false)       // Assume executions will go well (see note in finally block)
       this.running = runningMan
       let runtime
@@ -316,16 +320,22 @@ class PyodideSectionsRunnerBase {
   /**Build the default configuration runtime to use to run the user's code.
    * */
   setupGlobalConfig(){
-    CONFIG.runningId   = this.id
-    CONFIG.running     = this.running.name
+    CONFIG.runningId = this.id
+    CONFIG.running   = this.running.name
     for(const prop of 'get del set keys'.split(' ')){
       const globName = prop+'Storage'
       const method   = `pyodide${ _.capitalize(prop) }Storage`
       globalThis[globName] = this[method].bind(this)    // Legacy behavior
     }
+    this.setupTerminalMessageRoutine()
   }
 
 
+  setupTerminalMessageRoutine(){
+    if(this.rotateTerminalMessage){
+      CONFIG.termMessage = this.termFeedbackFromPyodide.bind(this)
+    }
+  }
 
 
   pyodideGetStorage(key)  { noStorage() }   // sink
@@ -350,7 +360,7 @@ class PyodideSectionsRunnerBase {
 
   /**Method bound and assigned to CONFIG/terminalMessage, to automatically redirect python
    * calls to `terminal_message` to the current terminal's display. */
-  termFeedbackFromPyodide(){}                     // sink
+  termFeedbackFromPyodide(){}   // sink
 
   /**Generic steps to apply at the very end of a `runPythonCodeWithOptionsIfNoStdErr` call. */
   codeSnippetEndFeedback(runtime, step, code){}   // sink
@@ -436,6 +446,8 @@ class PyodideSectionsRunnerBase {
           code,
           purgeTrace: runtime.purgeStackTrace,
           autoAssertExtraction: runtime.autoLogAssert,
+          purgeAssertionTrace: this.removeAssertionsStacktrace, // This is independent of the step running => not
+                                                                // tide to `runtime`, but directly to the runner.
         },
         asyncSecrets: testsStep === CONFIG.section.secrets,
         method: async (ctx)=>{
@@ -468,7 +480,7 @@ class PyodideSectionsRunnerBase {
     LOGGER_CONFIG.ACTIVATE && jsLogger("[CheckPoint] - PyodideSectionsRunner teardownRuntime")
 
     await runtime.runWithCtx('post')
-    await this.handleMermaids(runtime)
+    this.renderPyodideGeneratedMermaids(runtime)
     await this._baseTeardownRuntime(runtime)
   }
 
@@ -480,10 +492,10 @@ class PyodideSectionsRunnerBase {
 
 
 
-  async handleMermaids(runtime){
+  renderPyodideGeneratedMermaids(runtime){
     LOGGER_CONFIG.ACTIVATE && jsLogger("[CheckPoint] - teardown mermaid")
 
-    if(!CONFIG.needMermaid || runtime.stopped){
+    if(!CONFIG.needMermaid || !CONFIG.calledMermaid){
       return
     }
 
@@ -493,12 +505,7 @@ class PyodideSectionsRunnerBase {
         +'Please contact the author of the exercice.'
       )
     }
-    try{
-      await mermaid.run()
-      // mermaid.run systematically throws an error, even on valid graphs...
-      // Worse: If mermaid.run({suppressErrors:true}) is used, nothing is rendered at all...
-      //        (I love JS...)
-    }catch(e){}
+    renderMermaidGraphs()
   }
 }
 
