@@ -722,15 +722,9 @@ class _TerminalHandler extends PyodideSectionsRunner {
   setupGlobalConfig(){
     super.setupGlobalConfig()
     CONFIG.cutFeedback = this.cutFeedback
-    this.setupTerminalMessageRoutine()
   }
 
 
-  setupTerminalMessageRoutine(){
-    if(this.rotateTerminalMessage){
-      CONFIG.termMessage = this.termFeedbackFromPyodide.bind(this)
-    }
-  }
 
 
   /**When a terminal is available, display stdout and errors in it, if any.
@@ -968,7 +962,9 @@ export class TerminalRunner extends _TerminalHandler {
       logConfig: {
         code: this.cmdOnTheRun,
         autoAssertExtraction: false,
-        purgeTrace: runtime.purgeStackTrace
+        purgeTrace: runtime.purgeStackTrace,
+        purgeAssertionTrace: this.removeAssertionsStacktrace, // This is independent of the step running => not
+                                                              // tide to `runtime`, but directly to the runner.
       },
     })
 
