@@ -98,6 +98,10 @@ class IdeStorageManager extends TerminalRunner {
     return storage
   }
 
+  getCodeFromStorage(){
+    return this.storage.code
+  }
+
 
   updateGenericStorageData(storage){
     storage.name   = this.pyName
@@ -408,7 +412,8 @@ export class IdeGuiManager extends IdeZipManager {
    * been applied before.
    * Note that the logic is rather intricate with super calls...
    * */
-  makeUpYourGui(){
+  makeUpYourGui(){      // Cap overload
+
     if(this.global.is(':hidden')) return false    // Nothing to do yet (tabbed content).
 
     const todo = [
@@ -644,6 +649,7 @@ export class IdeFullScreenGlobalManager extends IdeGuiManager {
    * */
   static buildBodyObserver(){
     new MutationObserver((records)=>{
+      if(!this.currentIde) return
 
       for(const record of records) {
 
@@ -697,7 +703,8 @@ export class IdeFullScreenGlobalManager extends IdeGuiManager {
   }
 
 
-  makeUpYourGui(){
+
+  makeUpYourGui(){        // Cap overload
 
     /* Add the logistic to track the use of ESC while the IDE search/replace tool is used
       ...
