@@ -27,8 +27,10 @@ import {
   decompressLZW,
   downloader,
   freshStore,
-  getTheme,
+  getIdeOptions,
+  perennialMathJaxUpdate,
   PythonError,
+  renderMermaidGraphs,
   sleep,
   subscribeWhenReady,
   txtFormat,
@@ -77,7 +79,7 @@ class IdeAceManager extends IdeSplitScreenManager {
   * */
   save(givenCode=""){   LOGGER_CONFIG.ACTIVATE && jsLogger("[Save]")
     const currentCode = givenCode || this.getCodeToTest()
-    this.setStorage({code: currentCode, hash:this.srcHash})
+    this.setStorage({code: currentCode, hash: this.srcHash})
   }
 
 
@@ -94,7 +96,7 @@ class IdeAceManager extends IdeSplitScreenManager {
     let exerciseCode = ""
 
     if(attemptLocalStorageExtraction){
-      exerciseCode = this.storage.code
+      exerciseCode = this.getCodeFromStorage()
     }
     if(!exerciseCode){
       exerciseCode = this._joinCodeAndPublicSections(this.userContent)
@@ -155,6 +157,8 @@ class IdeHistoryManager extends IdeAceManager {
     return (Date()+'').split(' GMT')[0].slice(-8)
   }
 
+  /**Store the data of the current run/validation to this.validations.
+   * */
   pushValidation(code, done){     // CodCap
     const time = this.getTime()
     this.validations.push([done, time, code])
@@ -285,13 +289,13 @@ class IdeFeedbackManager extends IdeHistoryManager {
    * Do not forget that any error related feedback has already been printed to the console, so
    * when it comes to feedback given in this method, it's only about a potential "final message".
    * */
-  handleRunOutcome(runtime, allowCountDecrease, code){
+  handleValidationOutcome(runtime, allowCountDecrease, code){
     const success      = !runtime.stopped
     const done         = success ? 1:-1
     const isDelayed    = this.profile === CONFIG.PROFILES.delayedReveal
     const someToReveal = this.corrRemsMask && this.hiddenDivContent && (!this.profile || isDelayed)
 
-    LOGGER_CONFIG.ACTIVATE && jsLogger("[CheckPoint] - handleRunOutcome")
+    LOGGER_CONFIG.ACTIVATE && jsLogger("[CheckPoint] - handleValidationOutcome")
     // console.log("[OutCome]", JSON.stringify({
     //   success, pof:this.profile, revealable, hidden:this.hiddenDivContent,
     //   allowCountDecrease: !!allowCountDecrease, mask:this.corrRemsMask, N:this.attemptsLeft
@@ -306,7 +310,10 @@ class IdeFeedbackManager extends IdeHistoryManager {
     }
 
     // Reveal if success and not delayed, or if attempts==0 (means last error, are last delayed)
-    const isRevelation =  someToReveal && (success && !isDelayed || this.attemptsLeft===0)
+    const isRevelation = someToReveal && (
+      this.attemptsLeft===0
+      || success && !isDelayed
+    )
 
     /*If success, a custom final message has to be built:
           - If the revelation already occurred, return no message at all
@@ -366,20 +373,23 @@ class IdeFeedbackManager extends IdeHistoryManager {
       LOGGER_CONFIG.ACTIVATE && jsLogger("[CheckPoint] - revealed!")
 
       const sol_div = $(this.solutionH)
-
-      if(this.isEncrypted){
-        const compressed = sol_div.text().trim()
-        const content    = compressed && decompressLZW(compressed, "ides.encrypt_corrections_and_rems")
-        sol_div.html(content)
-        this.data.is_encrypted = false  // Never decompress again
+      const content = sol_div.text().trim()
+      const isCompressed = content.includes(CONFIG.LZW)
+      if(isCompressed){
+        const unCompressed = decompressLZW(content, "ides.encrypt_corrections_and_rems")
+        sol_div.html(unCompressed)
       }
       sol_div.attr('class', '')         // Unhide
       this.hiddenDivContent = false     // Never reveal again (last in case of errors...)
 
-      // Enforce formatting
-      if(waitForMathJax){
+      // Enforce formattings of REM contents
+      renderMermaidGraphs()
+
+      if(!waitForMathJax){
+        perennialMathJaxUpdate()
+      }else{
         subscribeWhenReady(
-          "revealOnLoad", mathJaxUpdate, {
+          "revealOnLoad", perennialMathJaxUpdate, {
             waitFor: ()=>CONFIG.subscriptionReady.MathJax,
             runOnly: true,
             delay: 100,
@@ -387,17 +397,7 @@ class IdeFeedbackManager extends IdeHistoryManager {
                             // properly if pyodide is starting "too soon".
           }
         )
-      }else{
-        mathJaxUpdate()
       }
-
-      /* DOESN'T WORK
-      // Enforce mermaid rendering if any
-      try{
-        // Need to make an async call, but i don't want to make this method an async one...
-        setTimeout(async ()=>console.log('yup?') || await mermaid.run())
-      }catch(e){}
-      //*/
     }
   }
 
@@ -477,12 +477,11 @@ class IdeFeedbackManager extends IdeHistoryManager {
  * */
 class IdeRunnerLogic extends IdeFeedbackManager {
 
-
   /**Is the current action "running the public tests"? (legacy) */
   isPlaying(){  return this.running.isPlaying }
 
   /**Is the current action "running the validation tests"? (legacy) */
-  isChecking(){ return this.running.isChecking }
+  isValidating(){ return this.running.isValidating }
 
   /**Does nothing, but catch any call on the parent class, which would raise an error
    * because prefillTerm is undefined, for IDEs.
@@ -593,7 +592,7 @@ class IdeRunnerLogic extends IdeFeedbackManager {
 
       // ... but decrease the number attempts and run teardown if this was AssertionError.
       if(runtime.isAssertErr){
-        this.handleRunOutcome(runtime, decrease_count, code)
+        this.handleValidationOutcome(runtime, decrease_count, code)
       }
 
     }else{
@@ -623,7 +622,7 @@ class IdeRunnerLogic extends IdeFeedbackManager {
 
       // Reveal solution and REMs on success, or if the counter reached 0 and the corr&REMs content
       // is still hidden, then prepare an appropriate revelation message if needed (`finalMsg`).
-      this.handleRunOutcome(runtime, decrease_count, code)
+      this.handleValidationOutcome(runtime, decrease_count, code)
     }
   }
 
@@ -660,9 +659,9 @@ class IdeRunnerLogic extends IdeFeedbackManager {
     LOGGER_CONFIG.ACTIVATE && jsLogger("[CheckPoint] - corr_btn start")
 
     this._corrConfigSwap = {            // (not defined in the constructor. MEH!  xp )
-      codeGetter:   this.getCodeToTest,
-      profile:      this.profile,
       currentCode:  this.getCodeToTest(),
+      codeGetter:   this.getCodeToTest,
+      profile:      this.profile,       // The profile/MODE has to be ignored for the CORR button => store original
     }
 
     this.getCodeToTest = ()=>this.corrContent
@@ -677,9 +676,10 @@ class IdeRunnerLogic extends IdeFeedbackManager {
     try{
       out = await this.teardownRuntimeIDE(runtime)
     }finally{
-      const {codeGetter, profile, currentCode} = this._corrConfigSwap
-      this.getCodeToTest = codeGetter
+      // Restore normal values:
+      const {currentCode, codeGetter, profile} = this._corrConfigSwap
       this.data.profile  = profile
+      this.getCodeToTest = codeGetter
       this.save(currentCode)     // Ensure the corr content doesn't stay stored in localStorage
     }
     return out
@@ -783,29 +783,9 @@ export class IdeRunner extends IdeRunnerLogic {
   /**Create and setup the ACE editor for the current Ide instance.
    * */
   setupAceEditor() {
-
-    // https://github.com/ajaxorg/ace/wiki/Configuring-Ace
-    const options = {
-        autoScrollEditorIntoView: false,
-        copyWithEmptySelection:   true,               // active alt+flèches pour déplacer une ligne, aussi
-        enableBasicAutocompletion:true,
-        enableLiveAutocompletion: false,
-        enableSnippets:           true,
-        tabSize:                  4,
-        useSoftTabs:              true,               // Spaces instead of tabs
-        navigateWithinSoftTabs:   false,              // this is _fucking_ actually "Atomic Soft Tabs"...
-        printMargin:              false,              // hide ugly margins...
-        maxLines:                 this.maxIdeLines,
-        minLines:                 this.minIdeLines,
-        mode:                     "ace/mode/python",
-        theme:                    getTheme(),
-        fontSize:                 CONFIG.editorFontSize,
-        fontFamily:               [CONFIG.editorFontFamily, "Monaco", "Menlo", "Ubuntu Mono", "Consolas",
-                                   "Source Code pro", "source-code-pro", "monospace"],   // Fix apple troubles...
-    }
-
-    this.editor = ace.edit(this.id, options);
-    this.gutter = this.global.find('div.ace_gutter-layer')
+    const options = getIdeOptions(this)
+    this.editor   = ace.edit(this.id, options);
+    this.gutter   = this.global.find('div.ace_gutter-layer')
 
     if(CONFIG._devMode) CONFIG.editors[this.id] = this.editor
 
@@ -830,7 +810,11 @@ export class IdeRunner extends IdeRunnerLogic {
         exec: this.runners.validate.asEvent,
       })
     }
+    this._buildAutoSaveOnKeyStroke()
+  }
 
+
+  _buildAutoSaveOnKeyStroke(){
     // Content of the editor is saved every `CONFIG.ideKeyStrokesSave` keystrokes:
     let nChange = 0;
     this.editor.addEventListener("input", _=>{
@@ -842,6 +826,9 @@ export class IdeRunner extends IdeRunnerLogic {
   }
 
 
+  buildCorrStuff(){ return CONFIG.inServe }
+
+
 
   bindIdeButtons(){
 
@@ -861,6 +848,8 @@ export class IdeRunner extends IdeRunnerLogic {
 
     // Bind all buttons below the IDE
     const ideThis = this
+    const corrStuff = this.buildCorrStuff()
+
     this.global.find("button").each(function(){
 
       const btn  = $(this)
@@ -878,16 +867,16 @@ export class IdeRunner extends IdeRunnerLogic {
                           )
                           callback = ideThis.runners.validate.asEvent ; break
 
-        case 'download':  callback = _=>ideThis.download() ; break
-        case 'upload':    callback = _=>ideThis.upload() ; break
+        case 'download':  callback = _=>{ ideThis.download() } ; break
+        case 'upload':    callback = _=>{ ideThis.upload() } ; break
 
-        case 'restart':   callback = _=>ideThis.restart() ; break
-        case 'save':      callback = _=>{ideThis.save(); ideThis.focusEditor()} ; break
+        case 'restart':   callback = _=>{ ideThis.restart() } ; break
+        case 'save':      callback = _=>{ ideThis.save(); ideThis.focusEditor() } ; break
         case 'zip':       callback = ideThis.buildZipExportsToolsAndCbk(btn) ; break
 
-        case 'corr_btn':  if(!CONFIG.inServe) return;
+        case 'corr_btn':  if(!corrStuff) return;
                           callback = ideThis.runners.validateCorr.asEvent ; break
-        case 'show':      if(!CONFIG.inServe) return;
+        case 'show':      if(!corrStuff) return;
                           callback = ()=>ideThis.revealSolutionAndRems() ; break
 
         default:          throw new Error(`Y'should never get there, mate... (${ kind })`)
@@ -993,19 +982,27 @@ export class IdeRunner extends IdeRunnerLogic {
 
   /**Download the current content of the editor to the download folder of the user.
    * */
-  download(){   LOGGER_CONFIG.ACTIVATE && jsLogger("[Download]")    // CodCap
+  download(content=null){   LOGGER_CONFIG.ACTIVATE && jsLogger("[Download]")    // CodCap
 
-    let ideContent = this.getCodeToTest() + "" // enforce stringification in any case
+    let ideContent = content ?? this.getCodeToTest() + "" // enforce stringification in any case
     downloader(ideContent, this.pyName)
     this.focusEditor()
   }
 
 
+
+
   /**Reset the content of the editor to its initial content, and reset the localStorage for
    * this editor on the way.
+   *
+   * @returns; true if the user confirmed the reset.
    * */
   restart(){    LOGGER_CONFIG.ACTIVATE && jsLogger("[Restart]")   // CodCap
 
+    if(!window.confirm(CONFIG.lang.restartConfirm.msg)){
+      return false
+    }
+
     this.setStartingCode({extractFromLocalStorage: false})
     this.storage = freshStore("", {}, this)
     localStorage.removeItem(this.id)
@@ -1020,6 +1017,7 @@ export class IdeRunner extends IdeRunnerLogic {
     this.terminal.clear()
     // clearPyodideScope()    // v4.2.0: no scope cleaning anymore.
     this.focusEditor()
+    return true
   }
 }
 
